'*************************************
'*  ZiCog 1.0_rc_6                   *
'*  An emulation of the Zilog z80    *
'*  (C) 2010 Michael Rychlik         *
'*************************************
'
'Licenceing:        - See end of file for MIT Licence conditions.
'
'Changes:           - See accompanying file CHANGES.TXT
'
'Credits:
'
'Bill Henning       - Inventor of the Large Memory Modle (LMM) technique for
'                     executing large PASM programs. Which makes ZiCog possible.
'
'Ray Rodick (Cluso) - Suggested the method of including multiple micro-op
'                     addresses and parameters into single dispatch table LONGs.
'                     Designer of the TriBlade and RamBlade external RAM solutions
'                     for the Prop on which this code runs very well.
'                     A complete CP/M computer in a matchbox !
'
'Brad Campbell      - Creator of the excellent BST Propeller IDE, thus making life
'                     for Mac and Linux users want to program the Prop much easier.
'
'James Moxam        - Designer of the DracBlade single board computer.
'                     A Prop CP/M machine for those who  can't see so well (big DIP chips).
'                     Probably the heaviest user/tester of ZiCog and CP/M.
'
'Juergen (PullMoll) - Author of the final working implementation of DAA and
'                     general Z80 encyclopedia.
'
'Toby Seckshund     - All round encouragement and inspiration.
'
'All on the Parallax Propeller fourum for a million suggestions and much patience in
'putting up with the interminably long ZyCog thread.
'
'
'Recent changes:
'                    Replaced PASM overlays with Large Memory Model (LMM) functions.
'                    Removed all PropDemoBoard support.
'                    New implementation of full Z80 DAA.
'
'N.B. This module must be compiled with BST or other Spin compiler that
'     supports conditional compilation and the @@@ operator.


CON
#ifdef TriBladeProp
#else
#warn No HW platform type defined
#endif


#ifdef CPU_Z80
#elseifdef CPU_8080
#else
#warn No CPU type defined
#endif

  'Z80 flag bits
  sign_bit   = %10000000
  zero_bit   = %01000000
  aux_bit    = %00010000
  parity_bit = %00000100
  neg_bit    = %00000010
  carry_bit  = %00000001

  bit_0_mask = %00000001
  bit_1_mask = %00000010
  bit_2_mask = %00000100
  bit_3_mask = %00001000
  bit_4_mask = %00010000
  bit_5_mask = %00100000                          
  bit_6_mask = %01000000
  bit_7_mask = %10000000


  'I/O control block commands
  io_cmd_out   = $01 
  io_cmd_in    = $02 
  io_cmd_break = $03

#ifdef banked_memory
  'Memory bank select port.
  bank_sel_port = $40
#endif

'The following taken from the SIMH Altair simulator cfgccp.lib
#ifdef TriBladeProp or Dracblade
  msize   = 64           'Adjust per installed system memory (K bytes)
#else
  msize   = 25           'Adjust per installed system memory (K bytes)
#endif

#ifdef Dracblade
  bioslen = $F00         'Adjust as bios changes are made (if necessary)
  ccplen  = $800         'cp/m constant
  bdoslen = $e00         'cp/m constant

' cp/m image size (rounded up to next 1k boundary)
cpmlen = (ccplen + bdoslen + bioslen + $3ff) / $400

ccpph   =     (msize - cpmlen) * 1024         'ccp start address
bdosph  =     ccpph + ccplen                  'bdos start address
biosph  =     ccpph + ccplen + bdoslen        'bios start address

tpa     =     $100                            'CP/M's Transient Program Area

bdos    = bdosph + 6                          'Entry point for BDOS
#endif blahblah

VAR
  long  cog
  long  cur_op                                'Current opcode, about to be executed


PUB start(cpu_params) : okay | reg_base, io_base
'' Start CPU simulator - starts a cog
'' returns false if no cog available
''
''   cpu_par_params is a pointer to CPU parameter list
                                                                                           
  stop

  reg_base := LONG[cpu_params]
  c_reg    := reg_base + 0
  b_reg    := reg_base + 1
  e_reg    := reg_base + 2
  d_reg    := reg_base + 3
  l_reg    := reg_base + 4
  h_reg    := reg_base + 5
  f_reg    := reg_base + 6
  a_reg    := reg_base + 7
  im_reg   := reg_base + 16
  ix_reg   := reg_base + 18
  iy_reg   := reg_base + 20
  sp_reg   := reg_base + 22
  pc_reg   := reg_base + 24

#ifdef TriBladeProp or Dracblade
  ram_size := $FB00 '(1024 * msize) - $100
#else
#warn RAM size not defined
#endif

  io_base    := LONG[cpu_params + 4]
  io_command := io_base
  io_port    := io_base + 4
  io_data    := io_base + 8

  'Initilize registers
  pc := WORD[pc_reg]
  flags := WORD[f_reg]
  sp := WORD[sp_reg]

#ifdef SingleStep
  cur_op_addr := @cur_op
#endif

  okay := cog := cognew(@entry, 0) + 1                     'Start emulator in a new COG

#ifdef Dracblade
PUB dma_write (dma_address, hub_address)
' Write 128 bytes from the given hub address to a
' virtual Direct Memory Access buffer area in Z80 RAM.
'
' dma_address - Z80 RAM address of the DMA buffer
' hub_address - HUB RAM address of the bytes to write
  BYTEMOVE (@ram_z80 + dma_address, hub_address, 128)


PUB dma_read (dma_address, hub_address)
' Read 128 bytes from the given hub address to a
' virtual Direct Memory Access buffer area in Z80 RAM.
'
' dma_address - Z80 RAM address of the DMA buffer
' hub_address - HUB RAM address of the bytes to write
  BYTEMOVE (hub_address, @ram_z80 + dma_address,  128)  
#endif
PUB stop

'' Stop CPU simulator - frees a cog

  if cog
    cogstop(cog~ - 1)


PUB get_current_opcode
' Returns the opcode about to be exected when single stepping
  return cur_op

'---------------------------------------------------------------------------------------------------------
DAT

ram_z80
'---------------------------------------------------------------------------------------------------------

DAT
                        org     0
entry
#ifdef TriBladeProp
'the following are also used as variables later...
'The TriBladeProp Blade #2 driver presets the latch to enable SRAM -CE(U23)=0 and hold LE=0
datax                   mov     outa, ram_latched          '-WE=1 -OE=1, all other bits=0
outx                    mov     dira, ram_dir_read         ' setup direction to read
#endif

#ifdef HEARTBEAT
                        mov     dira, heartbeat_pin        'Set Pins to output
#endif
'---------------------------------------------------------------------------------------------------------

'Instruction fetch loop starts here
disable_interrupts
enable_interrupts
fetch

#ifdef HEARTBEAT
                        xor     outa, heartbeat_pin        'N.B. If using heartbeat for KIPS measurement                                                           'remember to multiply the frequency by 2.
#endif
                        mov     dispatch_tab_addr, dispatch_tab_00_addr

dispatch                mov     address, pc
                        call    #read_memory_byte          'Read the instruction byte there 

#ifdef SingleStep
                        wrbyte  data_8, cur_op_addr        'Make opcode available to Spin break routine
                        call    #run_break_lmm
                        rdbyte  data_8, cur_op_addr        'Get the opcode back again
#endif
                        add     pc, #1                     'Increment the program counter

dispatch_2              shl     data_8, #2                 'Mul by 4 to get a LONG offset into the table
                        add     data_8, dispatch_tab_addr  'Index into table
                        rdlong  vector, data_8             'Get vector from table
vect_1                  jmp     vector                     'Jump(indirect) to the instruction handler

vect_2
vect_3                  shr     vector, #9                 'shift to 2nd (or 3rd) vector
                        jmp     vector                     'Jump(indirect) to the instruction handler


pc                      long    0                          'Z80's Pogram Conter
sp                      long    0                          'Z80's Stack pointer
flags                   long    0                          'Z80's flags
xy_reg                  long    0                          'Points to IX or IY depending on DD or FD prefix

#ifdef HEARTBEAT
heartbeat_pin           long    %00000000_00000001_00000000_00000000
#endif

dispatch_tab_addr       long    0
dispatch_tab_00_addr    long    @@@dispatch_table_00

#ifdef CPU_Z80
dispatch_tab_cb_addr    long    @@@dispatch_table_cb
dispatch_tab_dd_addr    long    @@@dispatch_table_dd
dispatch_tab_ddcb_addr  long    @@@dispatch_table_ddcb
dispatch_tab_ed_addr    long    @@@dispatch_table_ed
#endif

#ifdef SingleStep
cur_op_addr             long    0-0
#endif

vector                  long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Data Source Functions

get_b                   rdbyte  data_8, b_reg
                        jmp     #vect_2

get_c                   rdbyte  data_8, c_reg
                        jmp     #vect_2

get_d                   rdbyte  data_8, d_reg
                        jmp     #vect_2

get_e                   rdbyte  data_8, e_reg
                        jmp     #vect_2

get_h                   rdbyte  data_8, h_reg
                        jmp     #vect_2

get_l                   rdbyte  data_8, l_reg
                        jmp     #vect_2

get_a                   rdbyte  data_8, a_reg
                        jmp     #vect_2

get_bc                  rdword  data_16, b_reg
                        jmp     #vect_2
                        
get_de                  rdword  data_16, d_reg
                        jmp     #vect_2

get_hl                  rdword  data_16, h_reg
                        jmp     #vect_2

get_af                  rdbyte  data_16, a_reg
                        shl     data_16, #8
                        or      data_16, flags
                        jmp     #vect_2

get_ixy                 rdword  data_16, xy_reg
                        jmp     #vect_2

get_sp                  mov     data_16, sp
                        jmp     #vect_2

get_pc                  mov     data_16, pc
                        jmp     #vect_2

get_m
get_memory_hl           rdword  address, h_reg
get_memory_addr         call    #read_memory_byte
                        jmp     #vect_2

get_memory_bc           rdword  address, b_reg
                        jmp     #get_memory_addr

get_memory_de           rdword  address, d_reg
                        jmp     #get_memory_addr


#ifdef CPU_Z80
address_frm_xy_disp     mov     address, pc                'Get displacement from immediate byte
                        call    #read_memory_byte
                        add     pc, #1
                        shl     data_8, #24                'Sign extent the displacement
                        sar     data_8, #24
                        rdword  address, xy_reg            'Get the address from IX or IY index register
                        shl     address, #16               'Sign extend the address
                        sar     address, #16
                        adds    address, data_8            'Twos complement addition of address and displacement
address_frm_xy_disp_ret ret

get_mem_xy_disp         call    #address_frm_xy_disp
                        jmp     #get_memory_addr

address_stash           long    0

#endif

get_immediate_byte      mov     address, pc                'Read a BYTE from location given by "pc"
                        add     pc, #1                     'of Z80 RAM into byte data_8 and increment pc
                        jmp     #get_memory_addr

 
get_immediate_word      mov     address, pc                'Read a WORD from location given by "pc"
                        call    #read_memory_word          'of Z80 RAM into byte data_16 and pc = pc + 2
                        add     pc, #2
                        jmp     #vect_2

get_tos                 mov     address, sp
                        call    #read_memory_word
                        jmp     #vect_2


test_flag_true          mov     alu, vector
                        shr     alu, #18
                        test    flags,alu wz
                        jmp     #vect_2
                        

test_flag_false         mov     alu, vector
                        shr     alu, #18
                        xor     flags, #$FF
                        test    flags, alu wz
                        xor     flags, #$FF
                        jmp     #vect_2


return_conditional
              if_z      jmp     #fetch

return_unconditional    mov     address, sp                'Get return address from the stack
                        call    #read_memory_word
                        mov     pc, data_16                'Jump to it
                        add     sp, #2                     'Complete the "pop"
                        jmp     #fetch

jump_rel_conditional
              if_z       add pc,#1
              if_z       jmp #fetch

jump_rel_unconditional  mov    address, pc                 'Get target displacement from immediate byte
                        call   #read_memory_byte
                        add    pc, #1
                        shl    data_8, #24                 'Offset is two's comp. so sign extend it.
                        sar    data_8, #24
                        add    pc, data_8                  'and add to the PC.
                        jmp    #fetch
                        
jump_conditional        
              if_z      add     pc, #2
              if_z      jmp     #fetch
              

jump_unconditional      mov     address, pc                'Get target address from immediate word
                        call    #read_memory_word
                        mov     pc, data_16
                        jmp     #fetch                     'and jump to it

call_conditional
              if_z      add     pc, #2
              if_z      jmp     #fetch

call_unconditional      mov     data_16, pc
                        add     data_16, #2
                        call    #push
                        jmp     #jump_unconditional

dec_jmp_nz              rdbyte  data_8, b_reg              'Decrement B register
                        sub     data_8, #1 wz
                        wrbyte  data_8, b_reg
              if_nz     jmp     #jump_rel_unconditional    'Jump if not zero
                        add     pc, #1                     'Else skip immediate byte
                        jmp     #fetch                        
        

restart                 mov     data_16, pc                'Push return aaddress
                        call    #push
                        shr     vector, #9                 'Get restart target address from vector
                        mov     pc, vector                 'and jump there.
                        jmp     #fetch
                        

push                    sub     sp, #2
                        mov     address, sp
                        call    #write_memory_word
push_ret                ret


push_                   call    #push
                        jmp     #fetch


pop                     mov     address, sp
                        call    #read_memory_word
                        add     sp, #2                     'Complete the "pop"
pop_ret                 ret


pop_                    call    #pop
                        jmp     #vect_2


#ifdef CPU_Z80
z80_CB                  mov     dispatch_tab_addr, dispatch_tab_cb_addr  'Z80, CB Prefix
                        jmp     #dispatch 


z80_DD                  mov     xy_reg, ix_reg             'Point XY reg to IX reg for DD prefix ops
                        mov     dispatch_tab_addr, dispatch_tab_dd_addr  'Z80, DD Prefix
                        jmp     #dispatch 


z80_DDCB                mov     address_stash, address     'Z80 DDCB prefix
                        mov     address, pc
                        call    #read_memory_byte          'Read the instruction byte there
                        add     pc, #1                     'Increment the program counter
                        shr     data_8, #3                 'DDCB table only has 32 entries so discard 3 low bits
                        mov     dispatch_tab_addr, dispatch_tab_ddcb_addr
                        jmp     #dispatch_2


z80_ED                  mov     address, pc                'Z80, ED Prefix
                        call    #read_memory_byte          'Read the instruction byte there 
                        add     pc, #1                     'Increment the program counter
                        sub     data_8, #$40 wc            'ED dispatch table starts at $40  FIXME Should we range check ED opcodes?
                        mov     dispatch_tab_addr, dispatch_tab_ed_addr
                        jmp     #dispatch_2 


z80_FD                  'FIXME Set something here to select IY
                        mov     xy_reg, iy_reg             'Point XY reg to IY reg for FD prefix ops
                        mov     dispatch_tab_addr, dispatch_tab_dd_addr 'Z80, FD Prefix
                        jmp     #dispatch 
#endif

store_a_direct          mov     address, data_16
                        rdbyte   data_8, a_reg
                        call    #write_memory_byte
                        jmp     #fetch


load_a_direct           mov     address, data_16
                        call    #read_memory_byte
                        wrbyte  data_8, a_reg     
                        jmp     #fetch


store_word_direct       mov     alu, data_16               'Save the word for a while
                        mov     address, pc                'Read a WORD from location given by "pc"
                        call    #read_memory_word          'of Z80 RAM into byte data_16 and pc = pc + 2
                        add     pc, #2
                        mov     address, data_16           'Use the immediate word as target address
                        mov     data_16, alu               'Get the word back again                        
                        call    #write_memory_word         'and store it
                        jmp     #fetch


load_word_direct        mov     address, pc                'Read a WORD from location given by "pc"
                        call    #read_memory_word          'of Z80 RAM into byte data_16 and pc = pc + 2
                        add     pc, #2
                        mov     address, data_16           'Use the immediate word as target address
                        call    #read_memory_word          'and load the word.
                        jmp     #vect_2

z80_exx
in_c
out_c

'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Data Destination Functions

put_b                   wrbyte  data_8, b_reg
                        jmp     #fetch

put_c                   wrbyte  data_8, c_reg
                        jmp     #fetch

put_d                   wrbyte  data_8, d_reg
                        jmp     #fetch

put_e                   wrbyte  data_8, e_reg
                        jmp     #fetch

put_h                   wrbyte  data_8, h_reg
                        jmp     #fetch

put_l                   wrbyte  data_8, l_reg
                        jmp     #fetch

put_a                   wrbyte  data_8, a_reg   
                        jmp     #fetch
                        
put_bc                  wrword  data_16, b_reg
                        jmp     #fetch

put_de                  wrword  data_16, d_reg
                        jmp     #fetch

put_hl                  wrword  data_16, h_reg
                        jmp     #fetch

put_af                  mov     flags, data_16
                        and     flags, #$FF
                        shr     data_16, #8
                        wrbyte  data_16, a_reg
                        jmp     #fetch

put_ixy                 wrword  data_16, xy_reg
                        jmp     #fetch

put_sp                  mov     sp, data_16
                        jmp     #fetch

put_pc                  mov     pc, data_16
                        jmp     #fetch

put_m                 
put_memory_hl           rdword  address, h_reg
put_memory_addr         call    #write_memory_byte
                        jmp     #fetch

put_memory_bc           rdword  address, b_reg
                        jmp     #put_memory_addr

put_memory_de           rdword  address, d_reg
                        jmp     #put_memory_addr

#ifdef CPU_Z80
put_mem_xy_disp         mov     alu, data_8                'Save data_8 from corruption
                        call    #address_frm_xy_disp
                        mov     data_8, alu                'Get data_8 back again
                        jmp     #put_memory_addr

'FIXME is this the best way to do this ? This instruction does not fit our vector frame work.
ld_ix_disp_byte         call    #address_frm_xy_disp       'Make an address out of index reg and displacement
                        mov     address_stash, address     'Stash the address away for a while
                        mov     address, pc                'Get the immediate data byte into data_8
                        call    #read_memory_byte
                        add     pc, #1
                        
put_memory_sa           mov     address, address_stash     'Write data_8 out to stashed address
                        jmp     #put_memory_addr
#endif

put_tos                 mov     address, sp
                        call    #write_memory_word
                        jmp     #fetch


'---------------------------------------------------------------------------------------------------------
'---------------------------------------------------------------------------------------------------------
'Memory Access Functions

#ifdef TriBladeProp
'                                          +-------- -CS (U26 FLASH)   -+
'                                          |+------- -CS (J22 microSD)  +- Latched pins...
'                                          ||+------ -RST3              |    passes when LE=1
'             (P30) SO ---+                |||+----- -RST1              |    latches on  LE low going edge
'             (P31) SI --+|                ||||+---- -CE (U24 SRAM-hi)  |    latched on  LE=0
'                        ||+---- -WE(SDA)  |||||+--- -CE (U23 SRAM-lo)  |
'                        |||+--- -OE(SCL)  ||||||+-- A20                |
'                        ||||+--  LE       |||||||+- A19               -+
'                        |||||             ||||||||
'          P31...P0 --> %00xx0_xxxxxxxxxxx_xxxxxxxx_xxxxxxxx
ram_latched     long    %00110_00000000000_00000000_00000000 ' -WE=1 -OE=1 (LE=0)
ram_read        long    %00100_00000000000_00000000_00000000 ' -WE=1 -OE=0 (LE=0)
ram_write       long    %00010_00000000000_00000000_00000000 ' -WE=0 -OE=1 (LE=0)
ram_dir_read    long    %00110_11111111111_11111111_00000000 ' outputs WE, OE, A0-18, inputs D0-7
ram_dir_write   long    %00110_11111111111_11111111_11111111 ' outputs WE, OE, A0-18, D0-7
ram_dir_input   long    %00000_00000000000_00000000_00000000 ' all inputs (for Ram Disk access)
'                              +---------+ +------+ +------+
'                              A18......A8 A7....A0 D7....D0
'                                          Q7....Q0
'
'outx            long    0                                   'used as a temp address + WE/OE bits

'SRAM Initialisation: TriBladeProp Blade #2 Driver presets latch to -CE(U23)

'Read a BYTE from location given by "address" of Z80 RAM into byte data_8
ram_size                long    0-0                        'Length in bytes of available Z80 RAM

read_memory_byte
#ifdef banked_memory
                        cmp     address, common_ram wc     'Address in resident RAM area ?
              if_c      add     address, bank_offset       'No: Add offset to current memory bank in ext RAM
#endif
                        mov     outx, address
                        shl     outx,#8                    'shift address 8 bits
                        or      outx, ram_read             'add -WE=1 -OE=0
                        mov     outa, outx                 'pc (shifted 8 bits + -WE=1 -OE=0)
                        nop                                'delay for sram access time
                        mov     data_8, ina                'read SRAM
                        and     data_8, #$FF               'extract 8 bits
read_memory_byte_ret    ret

write_memory_byte
#ifdef banked_memory
                        and     address, mask_low_word     'Ensure address is within 64K
                        cmp     address, common_ram wc     'Address in resident RAM area ?
              if_c      add     address, bank_offset       'No: Add offset to current memory bank
#endif
                       ' cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              {if_b}    mov     outx, address
              {if_b}    shl     outx, #8                   'shift address 8 bits
              {if_b}    or      outx, ram_write            'add -WE=0 -OE=1
              {if_b}    mov     datax, data_8
              {if_b}    and     datax, #$FF                'ensure upper bytes=0
              {if_b}    or      outx, datax                'add data bits to be written
              {if_b}    mov     outa, outx                 'output -WE=0 -OE=1, address bits, data bits
              {if_b}    mov     dira, ram_dir_write        'set data bits to outputs
'             {if_b}    nop                                '25nS min delay (nothing reqd)
              {if_b}    or      outa, ram_latched          'set -WE=0->1 -OE=1->1  (end write)
              {if_b}    mov     dira, ram_dir_read         'set data bits to inputs
              {if_b}    mov     outa, ram_read             'set -WE=1 -OE=0
write_memory_byte_ret   ret

#ifdef banked_memory
common_ram              long    $C000                      'Address of common (resident) RAM in Z80 space.
bank_offset             long    0                          'Offset of the current memory bank in external RAM
#endif

#endif

#ifdef Dracblade
' **************************** Dr_Acula single propeller driver ****************
' three latches to access the ram chip - see main program for spin version
' uses variables address and data_8. skip banked memory and checking for out of bounds address
' will need two nop instructions in /rd and /wr to be sure 

Outx                    long    %00000000_00000000_00000000_00000000  ' temp variable                                   ' for temp use, same as n in the spin code
Direction138            long    %00000000_00000000_00001111_00000000 ' for reads so data lines are tristate till the read
LowAddress              long    %00000000_00000000_00000101_00000000 ' low address latch = xxxx010x and gate high xxxxxxx1
MiddleAddress           long    %00000000_00000000_00000111_00000000 ' middle address latch = xxxx011x and gate high xxxxxxx1
WriteEnable             long    %00000000_00000000_00000011_00000000 ' write enable xxxx001x and xxxxxxx0


read_memory_byte 
'#ifdef banked_memory
'                        cmp     address, common_ram wc     'Address in resident RAM area ?
'              if_c      add     address, bank_offset       'No: Add offset to current memory bank in ext RAM
'#endif
                        call #RamAddress                ' sets up the latches with the correct ram address
                        mov dira,Direction138           ' for reads so P0-P7 tristate till do read
                        mov outa,#$0                    ' set 000 low 138 read, low gate and P0-P7 are inputs
                        nop                             ' none does not work, 1 does work
                        mov data_8, ina                 ' read SRAM
                        and data_8, #$FF                ' extract 8 bits
                        or  outa,#$100                  ' set the gate high again
                        mov dira,#0                     ' tristate all pins
read_memory_byte_ret    ret

write_memory_byte
'#ifdef banked_memory
'                        and     address, mask_low_word     'Ensure address is within 64K
'                        cmp     address, common_ram wc     'Address in resident RAM area ?
'              if_c      add     address, bank_offset       'No: Add offset to current memory bank
'#endif

                        call #RamAddress                ' sets up the latches with the correct ram address
                        ' experiments both replacing outx with outa, and also using data_8 as the working variable instead of outx have both failed
                        mov outx,data_8                 ' get the byte to output
                        and outx, #$FF                  'ensure upper bytes=0
                        or outx,WriteEnable             ' or with correct 138 address which is the same as latchdirection2
                        mov outa,outx                   ' send it out
                        andn outa,#$100                 ' set gate low
                        nop                             ' NOP  needs at least one due to 138 propogation delay
                        or outa,#$100                   ' set it high again
                        mov dira,#0                     ' tristate all pins
write_memory_byte_ret   ret

RamAddress ' sets up the ram latches. Assumes high latch A16-A18 low so only accesses 64k of ram
                        mov dira,Direction138           ' set up the pins for programming latch chips
                        or  dira,#$FF                   ' set P0-P7 high
                        mov outx,address                ' get the address into a temp variable
                        and outx,#$FF                   ' mask the low byte
                        or  outx,LowAddress             ' or with 138 low address
'                        call #ToggleGate
                        mov outa,outx                   ' send it out
                        andn outa,#$100                 ' set gate low
                        or outa,#$100                   ' set it high again and repeat for the middle byte
                        mov outx,address                ' get the address into a temp variable
                        shr outx,#8                     ' shift right by 8 places
                        and outx,#$FF                   ' mask the low byte
                        or  outx,MiddleAddress          ' or with 138 middle address
'                        call #ToggleGate
                        mov outa,outx                   ' send it out
                        andn outa,#$100                 ' set gate low
                        or outa,#$100                   ' set it high again
RamAddress_ret          ret

'#ifdef banked_memory
'common_ram              long    $C000                      'Address of common (resident) RAM in Z80 space.
'bank_offset             long    0                          'Offset of the current memory bank in external RAM
'#endif

'ToggleGate              ' sends out outx
'                        mov outa,outx                   ' send it out
'                        andn outa,#$100                 ' set gate low
'                        or outa,#$100                   ' set it high again
'ToggleGate_ret          ret
#endif
'********************************************************************************   


'Read a WORD from location given by "address" of Z80 RAM into byte data_16
read_memory_word        call    #read_memory_byte          'Read the low byte
                        mov     data_16, data_8
                        ror     data_16, #8
                        add     address, #1                            
                        call    #read_memory_byte          'Read the high byte
                        or      data_16, data_8
                        rol     data_16, #8
read_memory_word_ret    ret

'Write the WORD data_16 to location given by "address"
write_memory_word       mov     data_8, data_16
                        call    #write_memory_byte         'Write the LOW byte
                        shr     data_8, #8
                        add     address, #1
                        call    #write_memory_byte
write_memory_word_ret   ret


'Z80 RAM read/write vars.
address                 long    0                          'Address param for read/write funcs
data_8                  long    0                          'Data bytes read into here
data_16                 long    0                          'Data words read into here
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Z80 Operation Functions

not_implemented
undocumented
halt
                        call    #run_break_lmm
                        jmp     #fetch

set_carry               or      flags, #carry_bit          'Set Carry Flag
                        andn    flags, #(aux_bit | neg_bit)
                        jmp     #fetch


complement_carry        test    flags, #carry_bit wz
                        muxnz   flags, #aux_bit
                        xor     flags, #carry_bit          'Complement Carry Flag
                        andn    flags, #neg_bit
                        jmp     #fetch


complement_byte         xor     data_8, #$FF
                        or      flags, #(aux_bit | neg_bit)
                        jmp     #vect_3

increment_byte
                        add     data_8, #1                 'Do the increment

                        andn    flags, #$FE                'Clear the flags to be set here

                        test    data_8, #$0F wz            'Set AUX flag when low nibble is zero
              if_z      or      flags, #aux_bit 

#ifdef CPU_Z80
                        cmp     data_8, #$80 wz
              if_z      or      flags, #parity_bit         'Set overflow flag when sign becomes negative
#endif

incdec_zero_sign        test    data_8, #$FF wz, wc
              if_z      or      flags, #zero_bit           'Set zero flag when result is zero

#ifdef CPU_8080
                        muxnc   flags, #parity_bit         'That "test" above gives parity in props carry flag
#endif
                        test    data_8, #128 wz            'Set sign flag from top bit.
                        muxnz   flags, #sign_bit           'NOTE: use code above for more accurate flag setting

                        jmp     #vect_3


decrement_byte          sub     data_8, #1                 'Do the increment
                        
                        andn    flags, #$FE                'Clear the flags to be set here
                        or      flags, #%00000010

                        mov     alu, data_8               
                        and     alu, #$0F
                        cmp     alu, #$0F wz
             if_z       or      flags, #aux_bit            'Set AUX flag when low nibble is $0F.

#ifdef CPU_Z80
                        cmp     data_8, #$7F wz
             if_z       or      flags, #parity_bit         'Set overflow flag when sign becomes positive
#endif

                        jmp     #incdec_zero_sign          'Set zero and sign flags as per increment.


alu_add                 rdbyte  alu, a_reg                 'Load A
                        mov     data_16, alu
                        add     data_16, data_8            'Result is in sum
                        mov     flags, #$00                'Clear all flags

fix_flags               mov     aux, alu
                        xor     aux, data_8
                        xor     aux, data_16
                        test    aux, #aux_bit wc
                        muxc    flags, #aux_bit

                        test    aux, #%1_0000_0000 wc
                        muxc    flags, #(parity_bit | carry_bit)
                        test    aux, #%1000_0000 wc
              if_c      xor     flags, #parity_bit

                        mov     data_8, data_16            'Post result and
                        jmp     #incdec_zero_sign          'sort out zero and sign flags


alu_add_with_carry      rdbyte  alu, a_reg                 'Load A

                        test    flags, #carry_bit wc       'Set prop's carry from Z80 carry flag
                        mov     data_16, alu
                        addx    data_16, data_8            'Do the op
                        mov     flags, #$00                'Clear all flags
                        jmp     #fix_flags


alu_sub                 rdbyte  alu, a_reg                 'Load A
                        mov     data_16, alu 
                        sub     data_16, data_8            'Do the op
                        mov     flags, #neg_bit            'Clear all flags excepy N
                        jmp     #fix_flags


alu_sub_with_borrow     rdbyte  alu, a_reg                 'Load A
                        test    flags, #carry_bit wc       'Set prop's carry from Z80 carry flag
                        mov     data_16, alu 
                        subx    data_16, data_8            'Do the op
                        mov     flags, #neg_bit            'Clear all flags excepy N
                        jmp     #fix_flags


zsp_flags               and     alu, #$FF wz, wc           'Set zero, sign and parity flags from alu content
                        muxz    flags, #zero_bit           'Set Z80 zero flag from props zero
                        muxnc   flags, #parity_bit         'That "and" above gives parity in props carry flag
                        test    alu, #128 wz               'Top bit is sign
                        muxnz   flags, #sign_bit
                        mov     data_8, alu
                        jmp     #vect_3


alu_and                 rdbyte  alu, a_reg                 'Load A
                        and     alu, data_8 wz, wc         'Do the AND and get flags
                        or      flags, #aux_bit            'AND always sets AUX flag (8085) 

alu_logic_flags         andn    flags, #(carry_bit | neg_bit) 'Logic ops always clear carry and N flags
                        jmp     #zsp_flags


alu_xor                 rdbyte  alu, a_reg                 'Load A
                        xor     alu, data_8 wz, wc         'Do the XOR and get flags
                        andn    flags, #aux_bit            'XOR always clears AUX flag
                        jmp     #alu_logic_flags


alu_or                  rdbyte  alu, a_reg                 'Load A
                        or      alu, data_8 wz, wc         'Do the OR and get flags
                        andn    flags, #aux_bit            'OR always clears AUX flag
                        jmp     #alu_logic_flags


alu_rlc                 shl     data_8, #1                 'Shift accumulator left by one
                        test    data_8, #%1_0000_0000 wz   'Move any 8 bit carry out bit into acc LSB
                        muxnz   data_8, #1
rot_end_1               muxnz   flags, #carry_bit          'Set carry flag from 8 bit overflow
                        andn    flags, #(aux_bit | neg_bit)
                        jmp     #vect_3

  
alu_rrc                 shr     data_8, #1 wc              'Shift accumulator right by one
                        muxc    data_8, #%10000000         'Move props carry to acc. MSB
rot_end_2               muxc    flags, #carry_bit          'Set carry flag from props carry
                        andn    flags, #(aux_bit | neg_bit)
                        jmp     #vect_3


alu_ral                 shl     data_8, #1                 'Shift accumulator left by one
                        test    flags, #carry_bit wz       'Set acc. LSB from carry flag
                        muxnz   data_8, #1                        
                        test    data_8, #%1_0000_0000 wz   'Set carry flag from 8 bit overflow
                        jmp     #rot_end_1


alu_rar                 shr     data_8, #1 wc              'Shift accumulator right by one
                        test    flags, #carry_bit wz       'Set acc. MSB from carry flag
                        muxnz   data_8, #%10000000
                        jmp     #rot_end_2


increment_word          add     data_16, #1
                        jmp     #vect_3


decrement_word          sub     data_16, #1
                        jmp     #vect_3


double_add_hl_cy        'FIXME for Z80
                        'FIXME Should set most flags !


double_add_hl           rdword  alu, h_reg
                        
double_add              mov     aux, alu                   'Take a copy of alu before operation for AUX carry determination

                        add     alu, data_16
                        test    alu, mask_bit_16 wz
                        muxnz   flags, #carry_bit
                        andn    flags, #%00000010

                        xor     aux, data_16
                        xor     aux,alu                    'Determine the auxillary carry flag
                        test    aux, aux_bit_shifted wc
                        muxc    flags, #aux_bit

                        mov     data_16, alu
                        jmp     #vect_3

aux_bit_shifted         long    (aux_bit << 8)

double_add_ixy          rdword  alu, xy_reg
                        jmp     #double_add


double_sub_hl_cy        'FIXME for Z80
                        'FIXME Should set most flags !


exchange                rdword address, h_reg
                        mov    data_8, data_16
                        mov    data_16, address
                        wrword  data_8, h_reg
                        jmp    #vect_3     


alu_sla
alu_sra
alu_srl




'---------------------------------------------------------------------------------------------------------
'Misc variables
alu                     long    0
aux                     long    0
nibble                  long    0
mask_bit_16             long    $0001_0000
mask_low_word           long    $0000_FFFF

'---------------------------------------------------------------------------------------------------------
'Pointers to Z80 registers in HUB RAM
b_reg                   long    0
c_reg                   long    0
d_reg                   long    0
e_reg                   long    0
h_reg                   long    0
l_reg                   long    0
m_reg                   'Note: m_reg is an alias for f_reg
f_reg                   long    0
a_reg                   long    0
im_reg                  long    0
ix_reg                  long    0
iy_reg                  long    0
sp_reg                  long    0
pc_reg                  long    0

'---------------------------------------------------------------------------------------------------------
'The I/O control block
io_command              long    0
io_port                 long    0
io_data                 long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Large Memory Model (LMM) virtual machine
lmm_run
lmm_fetch
#ifdef FAST_LMM
                        rdlong  :lmm_ins_0, lmm_pc
                        add     lmm_pc, #4
:lmm_ins_0              nop
                        rdlong  :lmm_ins_1, lmm_pc
                        add     lmm_pc, #4
:lmm_ins_1              nop
                        rdlong  :lmm_ins_2, lmm_pc
                        add     lmm_pc, #4
:lmm_ins_2              nop
#endif
                        rdlong  :lmm_ins_3, lmm_pc
                        add     lmm_pc, #4
:lmm_ins_3              nop
                        jmp     #lmm_fetch

lmm_pc                  long    0                           'LMM virtual machine program counter

#ifdef HAVE_LMM_CASE
' Jump though a jump table
lmm_case                shl     t, #2
                        add     pc, :t
                        jmp     #lmm_fetch
:t                      long    0

{
'lmm_case is used from lmm code like...
                        mov t,N
                        JMP #lmm_case
                        long   0addr
                        long   1addr
}
#endif

vector_lmm              mov     lmm_pc, vector              'Extract LMM function number from vector bits FIXME what
                        shr     lmm_pc, #7
                        and     lmm_pc, #%11111100
                        add     lmm_pc, lmm_function_tab_addr  'Offset into LMM function table
                        rdlong  lmm_pc, lmm_pc              'Get this LMM functions HUB address
                        jmp     #lmm_run                    'Start LMM VM on this function.

run_break_lmm           mov     lmm_pc, break_lmm_addr
                        jmp     #lmm_run
run_break_lmm_ret       ret

break_lmm_addr          long    @@@break_lmm
lmm_function_tab_addr   long    @@@lmm_function_table

'FIXME can we dispence with these somehow?
hl_temp                 long    0
de_temp                 long    0
bc_temp                 long    0
A                       long    0
adj_60                  long    0
adj_06                  long    0
'---------------------------------------------------------------------------------------------------------
                        fit       $1F0
'---------------------------------------------------------------------------------------------------------
CON
'LLM function numbers
'Note: These LMM function numbers must be the in the order of the
'      entries in the lmm_function_table.
daa_lmm_no     = $00
ldi_lmm_no     = $01
cpi_lmm_no     = $02
ini_lmm_no     = $03
outi_lmm_no    = $04
ldd_lmm_no     = $05
cpd_lmm_no     = $06
ind_lmm_no     = $07
outd_lmm_no    = $08
ldir_lmm_no    = $09
cpir_lmm_no    = $0A
inir_lmm_no    = $0B
otir_lmm_no    = $0C
lddr_lmm_no    = $0D
cpdr_lmm_no    = $0E
indr_lmm_no    = $0F
otdr_lmm_no    = $10
out_lmm_no     = $11
in_lmm_no      = $12
alu_neg_lmm_no = $13
alu_sla_lmm_no = $14
alu_sra_lmm_no = $15
alu_srl_lmm_no = $16

bit_reg_lmm_no = $17
bit_mem_lmm_no = $18
bit_idx_lmm_no = $19
set_reg_lmm_no = $1A
set_mem_lmm_no = $1B
set_idx_lmm_no = $1C
res_reg_lmm_no = $1D
res_mem_lmm_no = $1E
res_idx_lmm_no = $1F



{
{06}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'RLC     (IX+disp)      DDCBdisp06     ---
{0E}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'RRC     (IX+disp)      DDCBdisp0E     ---
{16}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'RL      (IX+disp)      DDCBdisp16     ---
{1E}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'RL      (IX+disp)      DDCBdisp1E     ---
{26}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'SLA     (IX+disp)      DDCBdisp26     ---
{2E}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'SRA     (IX+disp)      DDCBdisp2E     ---
{36}                    long    undocumented
{3E}                    long    get_stash          + (0{alu_bit}             << 9) + (put_memory_sa  << 18)              'SRL     (IX+disp)      DDCBdisp3E     ---
{46}                    long    get_stash          + (0{alu_bit}             << 9) + (put_memory_sa  << 18) + (0 << 27)  '*BIT     0,(IX+disp)    DDCBdisp46     Z flag <- NOT 0b
{4E}                    long    get_stash          + (0{alu_bit}             << 9) + (put_memory_sa  << 18) + (1 << 27)  '*BIT     1,(IX+disp)    DDCBdisp4E     Z flag <- NOT 1b
{56}                    long    get_stash          + (0{alu_bit}             << 9) + (put_memory_sa  << 18) + (2 << 27)  '*BIT     2,(IX+disp)    DDCBdisp56     Z flag <- NOT 2b
{5E}                    long    get_stash          + (0{alu_bit}             << 9) + (put_memory_sa  << 18) + (3 << 27)  '*BIT     3,(IX+disp)    DDCBdisp5E     Z flag <- NOT 3b
{66}                    long    get_stash          + (0{alu_bit}             << 9) + (put_memory_sa  << 18) + (4 << 27)  '*BIT     4,(IX+disp)    DDCBdisp66     Z flag <- NOT 4b
{6E}                    long    get_stash          + (0{alu_bit}             << 9) + (put_memory_sa  << 18) + (5 << 27)  '*BIT     5,(IX+disp)    DDCBdisp6E     Z flag <- NOT 5b
{76}                    long    get_stash          + (0{alu_bit}             << 9) + (put_memory_sa  << 18) + (6 << 27)  '*BIT     6,(IX+disp)    DDCBdisp76     Z flag <- NOT 6b
{7E}                    long    get_stash          + (0{alu_bit}             << 9) + (put_memory_sa  << 18) + (7 << 27)  '*BIT     7,(IX+disp)    DDCBdisp7E     Z flag <- NOT 7b
{86}                    long    get_stash          + (0{alu_res}             << 9) + (put_memory_sa  << 18) + (0 << 27)  '*RES     0,(IX+disp)    DDCBdisp86     0b <- 0
{8E}                    long    get_stash          + (0{alu_res}             << 9) + (put_memory_sa  << 18) + (1 << 27)  '*RES     1,(IX+disp)    DDCBdisp8E     1b <- 0
{96}                    long    get_stash          + (0{alu_res}             << 9) + (put_memory_sa  << 18) + (2 << 27)  '*RES     2,(IX+disp)    DDCBdisp96     2b <- 0
{9E}                    long    get_stash          + (0{alu_res}             << 9) + (put_memory_sa  << 18) + (3 << 27)  '*RES     3,(IX+disp)    DDCBdisp9E     3b <- 0
{A6}                    long    get_stash          + (0{alu_res}             << 9) + (put_memory_sa  << 18) + (4 << 27)  '*RES     4,(IX+disp)    DDCBdispA6     4b <- 0
{AE}                    long    get_stash          + (0{alu_res}             << 9) + (put_memory_sa  << 18) + (5 << 27)  '*RES     5,(IX+disp)    DDCBdispAE     5b <- 0
{B6}                    long    get_stash          + (0{alu_res}             << 9) + (put_memory_sa  << 18) + (6 << 27)  '*RES     6,(IX+disp)    DDCBdispB6     6b <- 0
{BE}                    long    get_stash          + (0{alu_res}             << 9) + (put_memory_sa  << 18) + (7 << 27)  '*RES     7,(IX+disp)    DDCBdispBE     7b <- 0
{C6}                    long    get_stash          + (0{alu_set}             << 9) + (put_memory_sa  << 18) + (0 << 27)  '**SET     0,(IX+disp)    DDCBdispC6     0b <- 1
{CE}                    long    get_stash          + (0{alu_set}             << 9) + (put_memory_sa  << 18) + (1 << 27)  '*SET     1,(IX+disp)    DDCBdispCE     1b <- 1
{D6}                    long    get_stash          + (0{alu_set}             << 9) + (put_memory_sa  << 18) + (2 << 27)  '*SET     2,(IX+disp)    DDCBdispD6     2b <- 1
{DE}                    long    get_stash          + (0{alu_set}             << 9) + (put_memory_sa  << 18) + (3 << 27)  '*SET     3,(IX+disp)    DDCBdispDE     3b <- 1
{E6}                    long    get_stash          + (0{alu_set}             << 9) + (put_memory_sa  << 18) + (4 << 27)  '*SET     4,(IX+disp)    DDCBdispE6     4b <- 1
{EE}                    long    get_stash          + (0{alu_set}             << 9) + (put_memory_sa  << 18) + (5 << 27)  '*SET     5,(IX+disp)    DDCBdispEE     5b <- 1
{F6}                    long    get_stash          + (0{alu_set}             << 9) + (put_memory_sa  << 18) + (6 << 27)  '*SET     6,(IX+disp)    DDCBdispF6     6b <- 1
{FE}                    long    get_stash          + (0{alu_set}             << 9) + (put_memory_sa  << 18) + (7 << 27)  '*SET     7,(IX+disp)    DDCBdispFE     7b <- 1
}




DAT
' Table of LMM function addresses
' Note: The LMM function numbers defined in CON must be in the order
'       of the function addresses here
lmm_function_table
{00}daa_lmm_            long    @@@daa_lmm
{01}ldi_lmm_            long    @@@ldi_lmm
{02}cpi_lmm_            long    @@@cpi_lmm
{03}ini_lmm_            long    @@@ini_lmm
{04}outi_lmm_           long    @@@outi_lmm
{05}ldd_lmm_            long    @@@ldd_lmm
{06}cpd_lmm_            long    @@@cpd_lmm
{07}ind_lmm_            long    @@@ind_lmm
{08}outd_lmm_           long    @@@outd_lmm
{09}ldir_lmm_           long    @@@ldir_lmm
{0A}cpir_lmm_           long    @@@cpir_lmm
{0B}inir_lmm_           long    @@@inir_lmm
{0C}otir_lmm_           long    @@@otir_lmm
{0D}lddr_lmm_           long    @@@lddr_lmm
{0E}cpdr_lmm_           long    @@@cpdr_lmm
{0F}indr_lmm_           long    @@@indr_lmm
{10}otdr_lmm_           long    @@@otdr_lmm
{11}out_lmm_            long    @@@out_lmm
{12}in_lmm_             long    @@@in_lmm
{13}alu_neg_lmm_        long    @@@alu_neg_lmm
{14}alu_sla_lmm_        long    @@@alu_sla_lmm
{15}alu_sra_lmm_        long    @@@alu_sra_lmm
{16}alu_srl_lmm_        long    @@@alu_srl_lmm

{17}bit_reg_lmm_        long    @@@bit_reg_lmm
{18}bit_mem_lmm_        long    @@@bit_mem_lmm
{19}bit_idx_lmm_        long    @@@bit_idx_lmm

{1A}set_reg_lmm_        long    @@@set_reg_lmm
{1B}set_mem_lmm_        long    @@@set_mem_lmm
{1C}set_idx_lmm_        long    @@@set_idx_lmm

{1D}res_reg_lmm_        long    @@@res_reg_lmm
{1E}res_mem_lmm_        long    @@@res_mem_lmm
{1F}res_idx_lmm_        long    @@@res_idx_lmm



DAT
'---------------------------------------------------------------------------------------------------------
'DAA LMM.
daa_lmm
'DAA implementation according to the (well tested) MAME Z80 cpu core.
                        rdbyte  A, a_reg                'Fetch the A register
                        mov     data_8, A               'Take a working copy
                        mov     nibble, A               'Get low 4 bits of A register
                        and     nibble, #$0F

                        mov     adj_60, #$60            'Set up adjustment values
                        mov     adj_06, #$06

                        test    flags, #neg_bit wz      'Negate adjustments if neg flag set
                if_nz   neg     adj_60, adj_60
                if_nz   neg     adj_06, adj_06

                        'If half carry or low nibble > 9 adjust by +/- 0x06
                        test    flags, #aux_bit wz
                        cmp     nibble, #$0A wc
           if_nc_or_nz  add     data_8, adj_06

                        'If carry or A reg greater than 0x99 adjust by +/- 0x60
                        test    flags, #carry_bit wz
                        cmp     A, #$9A wc
           if_nc_or_nz  add     data_8, adj_60

                        'Clear all flags except carry and neg
                        and     flags, #(carry_bit | neg_bit)

                        'Set carry if A reg greater than 0x99
                        cmp     A, #$9A wc
                  if_nc or      flags, #carry_bit       'FIXME we are testing A > 0x99 twice here.

                        'Set half carry of bit 4 changed
                        xor     A, data_8
                        and     A, #aux_bit
                        or      flags, A

                        'Set Sign Zero and Parity
                        and     data_8, #$FF wz, wc
                        muxz    flags, #zero_bit           'Set Z80 zero flag from Props zero
                        muxnc   flags, #parity_bit
                        test    data_8, #$80 wz
                        muxnz   flags, #sign_bit

                        wrbyte  data_8, a_reg              'Write the result to the A rgeister
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------
#ifdef CPU_Z80
'---------------------------------------------------------------------------------------------------------
'LDI LMM.
ldi_lmm                 rdword  address, h_reg             'data_8 = [HL]
                        call    #read_memory_byte
                        add     address, #1
                        wrword  address, h_reg             'HL = HL + 1

                        rdword  address, d_reg
                        call    #write_memory_byte         '[DE] = data_8
                        add     address, #1
                        wrword  address, d_reg             'DE = DE + 1

                        rdword  data_16, b_reg
                        sub     data_16, #1 wz
                        muxnz   flags, #parity_bit
                        wrword  data_16, b_reg             'BC = BC - 1
                        andn    flags, #(aux_bit  | neg_bit)
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'CPI LMM.
cpi_lmm                 jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'INI LMM.
' (HL) <- [C];
' B <- B-1;
' HL <- HL+1
ini_lmm                 jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'OUTI LMM.
'[C] <- (HL);
'B <- B-1;
'HL <- HL+1;
outi_lmm                rdbyte  data_8, c_reg              'Set the IO port from C
                        wrbyte  data_8, io_port            
                        rdword  address, h_reg             'Set port data from memory at HL
                        call    #read_memory_byte
                        wrbyte  data_8, io_data               
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT...
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'RR20090321 (pass I/O to another cog)
#endif
                        wrbyte  data_8, io_command         
:wait                   rdbyte  data_8, io_command wz      '...and wait for it to be completed.
              if_nz     sub     lmm_pc, #(2*4)             'jmp     #:wait 'in LMM
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'Take I/O from another cog
#endif
                        rdbyte  data_8, b_reg              'Update B reg
                        sub     data_8, #1
                        wrbyte  data_8, b_reg
                        add     address, #1                'Update HL reg
                        wrword  address, h_reg
                        jmp     #fetch                        
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LDD LMM.
ldd_lmm                 rdword  address, h_reg
                        call    #read_memory_byte
                        sub     address, #1
                        wrword  address, h_reg
                        rdword  address, d_reg
                        call    #write_memory_byte
                        sub     address, #1
                        wrword  address, d_reg
                        rdword  data_16, b_reg
                        sub     data_16, #1 wz
                        muxnz   flags, #parity_bit
                        wrword  data_16, b_reg
                        andn    flags, #(aux_bit  | neg_bit)
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'CPD LMM.
cpd_lmm                 jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'IND LMM.
ind_lmm                 jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'OUTD LMM.
' [C] <- (HL);
' B <- B-1;
' HL <- HL-1;
outd_lmm                rdbyte  data_8, c_reg              'Set the IO port from C
                        wrbyte  data_8, io_port            
                        rdword  address, h_reg             'Set port data from memory at HL
                        call    #read_memory_byte
                        wrbyte  data_8, io_data               
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT...
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'Pass I/O to another cog
#endif
                        wrbyte  data_8, io_command         
:wait                   rdbyte  data_8, io_command wz      '...and wait for it to be completed.
               if_nz    sub     lmm_pc, #(2*4)             'jmp     #:wait  'In LMM
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'Take I/O from another cog
#endif
                        rdbyte  data_8, b_reg              'Update B reg
                        sub     data_8, #1
                        wrbyte  data_8, b_reg
                        sub     address, #1                'Update HL reg
                        wrword  address, h_reg
                        jmp     #fetch                        
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LDIR LMM.
' (DE) <- (HL);
' HL   <- HL+1;
' DE   <- DE+1;
' BC   <- BC-1;
' repeat until BC = 0
ldir_lmm                rdword  hl_temp, h_reg
                        rdword  de_temp, d_reg
                        rdword  bc_temp, b_reg

:loop                   mov     address, hl_temp           'Move mem [HL] to mem [de]
                        call    #read_memory_byte
                        mov     address, de_temp
                        call    #write_memory_byte
                        add     hl_temp, #1                'Advance the pointers
                        add     de_temp, #1
                        sub     bc_temp, #1 wz             'Decrement counter...
                if_nz   sub     lmm_pc, #(8*4)             '...and maybe jmp to :loop

                        wrword  hl_temp, h_reg             'Update the registers
                        wrword  de_temp, d_reg
                        wrword  bc_temp, b_reg
                        andn    flags, #(parity_bit | aux_bit | neg_bit)

                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'CPIR LMM.
cpir_lmm                jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'INIR LMM.
inir_lmm                jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'OTIR LMM.
' [C] <- (HL);
' B <- B-1;
' HL <- HL+1;
otir_lmm
{FIXME
                        rdbyte  data_8, c_reg              'Set the IO port from C
                        wrbyte  data_8, io_port
                        rdword  address, h_reg
                        rdbyte  :b, b_reg 
:loop                   call    #read_memory_byte          'Set port data from memory at HL
                        wrbyte  data_8, io_data               
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT...
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'Pass I/O to another cog
#endif
                        wrbyte  data_8, io_command         
:wait                   rdbyte  data_8, io_command wz      '...and wait for it to be completed.
               if_nz    sub     lmm_pc, #(2*4)             'jmp     #:wait  'In LMM
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'Take I/O from another cog
#endif
                        add     address, #1                'Advance address pointer
                        djnz    :b, #:loop                 'Decrement counter and maybe loop
                        wrbyte  :b, b_reg                  'Update counter B reg
                        wrword  address, h_reg             'Update HL reg to next address
}
                        jmp     #fetch

:b                      long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'LDDR LMM.
' (DE) <- (HL);
' HL <- HL-1;
' DE <- DE-1;
' BC <- BC-1;
' repeat until BC = 0
lddr_lmm
{FIXME
                        rdword  :hl, h_reg
                        rdword  :de, d_reg
                        rdword  :bc, b_reg

:loop                   mov     address, :hl               'Move mem [HL] to mem [de]
                        call    #read_memory_byte
                        mov     address, :de
                        call    #write_memory_byte
                        sub     :hl, #1                    'Advance the pointers
                        sub     :de, #1
                        djnz    :bc, #:loop                'Decrement counter and maybe loop

                        wrword  :hl, h_reg                 'Update the registers
                        wrword  :de, d_reg
                        wrword  :bc, b_reg
                        andn    flags, #(parity_bit | aux_bit | %00010010)
}
                        jmp     #fetch

:hl                     long    0
:de                     long    0
:bc                     long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'CPDR LMM.
cpdr_lmm                jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'INDR LMM.
indr_lmm                jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'OTDR LMM.
' [C] <- (HL);
' B <- B-1;
' HL <- HL-1;
otdr_lmm
{FIXME
                        rdbyte  data_8, c_reg              'Set the IO port from C
                        wrbyte  data_8, io_port
                        rdword  address, h_reg
                        rdbyte  :b, b_reg
:loop                   call    #read_memory_byte          'Set port data from memory at HL
                        wrbyte  data_8, io_data               
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT...
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'Pass I/O to another cog
#endif
                        wrbyte  data_8, io_command         
:wait                   rdbyte  data_8, io_command wz      '...and wait for it to be completed.
               if_nz    sub     lmm_pc, #(2*4)             'jmp     #:wait  'In LMM
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'Take I/O from another cog
#endif
                        sub     address, #1                'Retard (?) address pointer
                        djnz    :b, #:loop                 'Decrement counter and maybe loop                        
                        wrbyte  :b, b_reg                  'Update counter B reg
                        wrword  address, h_reg             'Update HL reg to next address
}
                        jmp     #fetch
:b                      long    0                        
'---------------------------------------------------------------------------------------------------------

#endif

'---------------------------------------------------------------------------------------------------------
'OUT LMM.
out_lmm                 mov     address, pc                'Get IO port no. from immediate byte
                        call    #read_memory_byte
                        add     pc, #1

                        wrbyte  data_8, io_port
                        rdbyte  data_8, a_reg              'Get the output data from A reg
                        wrbyte  data_8, io_data            'Set the output data
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'Pass I/O to another cog
#endif
#ifdef BankedCPM
                        wrword  pc, pc_reg                 ' pass the cog pc to spin
                        wrbyte  flags, f_reg               ' pass the cog flags to spin
                        wrword  sp, sp_reg                 ' pass the cog sp to spin
#endif
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz      'Wait for the output done
              if_nz     sub     lmm_pc, #2*4               'JMP :wait in LMM
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'Take I/O from another cog
#endif
#ifdef BankedCPM
                        rdword  pc,pc_reg                 ' pass the spin pc to cog pc
                        rdbyte  flags,f_reg               ' pass the spin flags to cog flags
                        rdword  sp,sp_reg                 ' pass the spin sp to cog sp
#endif
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'IN LMM
in_lmm                  mov     address, pc                'Set IO port no. from immediate byte
                        call    #read_memory_byte
                        add     pc, #1
                        wrbyte  data_8, io_port
                        mov     data_8, #io_cmd_in         'Set I/O command to IN
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'Pass I/O to another cog
#endif
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz      'Wait for the input data
              if_nz     sub     lmm_pc, #2 * 4             'JMP :wait in LMM
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'Take I/O from another cog
#endif
                        rdbyte  data_8, io_data            'Read the input data to accumulator
                        wrbyte  data_8, a_reg
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

#ifdef CPU_Z80

'---------------------------------------------------------------------------------------------------------
'ALU NEG LMM
alu_neg_lmm             rdbyte  data_8, a_reg

                        mov     flags, #%00000010          'Clear all flags except N

                        test    data_8, #$0F wz            
              if_nz     or      flags, #aux_bit

                        cmp     data_8, #$80 wz
              if_z      or      flags, #parity_bit

                        cmp     data_8, #0 wz
              if_nz     or      flags, #carry_bit                        
              
                        neg     data_8, data_8              'Do the negation

                        wrbyte  data_8, a_reg

                        jmp     #incdec_zero_sign
'---------------------------------------------------------------------------------------------------------


'---------------------------------------------------------------------------------------------------------
'ALU SLA LMM.
alu_sla_lmm
                       ' shl     data_8, #1                 'FIXME shifts must set more flags !!!
                       ' test    data_8, #%1_0000_0000 wc   'Set props carry flag from 8 bit overflow
                       ' muxc    af, #carry_bit             'Set carry_bit from props cary
                       ' jmp     #vect_3
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ALU SRA LMM.
alu_sra_lmm
                       ' test    data_8, #%1000_0000 wz     'Set props zero from sign bit
                       ' shr     data_8, #1 wc              'Shift and get bit 0 to props carry
                       ' muxc    af, #carry_bit             'Set carry_bit from props cary
                       ' muxnz   data_8, #%1000_0000        'Set sign bit from props zero
                       ' jmp     #vect_3
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ALU SRL LMM.
alu_srl_lmm
                       ' shr     data_8, #1 wc              'Shift and get bit 0 to props carry
                       ' muxc    af, #carry_bit             'Set carry_bit from props cary
                       ' andn    data_8, #%1000_0000        'Ensure top bit of result is clear.                 
                       ' jmp     #vect_3
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BIT reg LMM.
bit_reg_lmm             shr     vector, #15                'Get bit mask from vector
                        mov     alu, vector                
                        and     alu, #$FF        'FIXME Is the required?
                        shr     vector, #8                 'Get register number from vector
                        add     vector, b_reg              'Add base address of register file
                        rdbyte  data_8, vector             'Get required register
                        test    data_8, alu wz             'Do the bit test
                        andn    flags, #$FE                'Clear all flags except carry
                        or      flags, #aux_bit            'Always set the AUX flag
                        and     alu, #sign_bit             'Set sign flag if bit 7 tested and is set TRUE.
              if_nz     or      flags, alu
              if_z      or      flags, #(zero_bit | parity_bit) 'Set ZERO and PARITY flags if bit was not set
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'BIT [HL] LMM.
bit_mem_lmm             shr     vector, #15                'Get bit mask from vector
                        mov     alu, vector
                        and     alu, #$FF              'FIXME Is the required?
                        rdword  address, h_reg             'Get byte from memory (HL)
                        call    #read_memory_byte
                        test    data_8, alu wz             'Do the bit test
                        rdword  address, h_reg             'Put byte to memory (HL)
                        call    #write_memory_byte
                        andn    flags, #$FE                'Clear all flags except carry
                        or      flags, #aux_bit            'Always set the AUX flag
                        and     alu, #sign_bit             'Set sign flag if bit 7 tested and is set TRUE.
              if_nz     or      flags, alu
              if_z      or      flags, #(zero_bit | parity_bit) 'Set ZERO and PARITY flags if bit was not set
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
bit_idx_lmm             shr     vector, #15                'Get bit mask from vector
                        mov     alu, vector
                        and     alu, #$FF          'FIXME Is the required?

                        mov     address, address_stash     'Retieve address IXY+disp and
                        call    #read_memory_byte          'read the byte from there
                        test    data_8, alu wz             'Do the bit test
                        call    #write_memory_byte
                        andn    flags, #$FE                'Clear all flags except carry
                        or      flags, #aux_bit            'Always set the AUX flag
                        and     alu, #sign_bit             'Set sign flag if bit 7 tested and is set TRUE.
              if_nz     or      flags, alu
              if_z      or      flags, #(zero_bit | parity_bit) 'Set ZERO and PARITY flags if bit was not set
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'RES reg LMM.
res_reg_lmm             shr     vector, #15                'Get bit mask from vector
                        mov     alu, vector                
                        and     alu, #$FF       'FIXME Is the required?
                        shr     vector, #8                 'Get register number from vector
                        add     vector, b_reg              'Add base address of register file
                        rdbyte  data_8, vector             'Get the required register
                        andn    data_8, alu                'Do the bit reset
:put_reg                wrbyte  data_8, vector             'Put the register back
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'RES [HL] LMM
res_mem_lmm             shr     vector, #15                'Get bit mask from vector
                        mov     alu, vector
                        and     alu, #$FF        'FIXME Is the required?
                        rdword  address, h_reg             'Get byte from memory (HL)
                        call    #read_memory_byte
                        andn    data_8, alu                'Do the bit reset
                        rdword  address, h_reg             'Put byte to memory (HL)
                        call    #write_memory_byte
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
res_idx_lmm
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'SET reg LMM.
set_reg_lmm             shr     vector, #15                'Get bit mask from vector
                        mov     alu, vector                
                        and     alu, #$FF        'FIXME Is the required?
                        shr     vector, #8                 'Get register number from vector
                        add     vector, b_reg              'Add base address of register file
                        rdbyte  data_8, vector             'Get the required register
                        or      data_8, alu                'Do the bit set
                        wrbyte  data_8, vector             'Put the register back
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------
'SET [HL] LMM.
set_mem_lmm             shr     vector, #15                'Get bit mask from vector
                        mov     alu, vector
                        and     alu, #$FF           'FIXME Is the required?
                        rdword  address, h_reg             'Get byte from memory (HL)
                        call    #read_memory_byte
                        or      data_8, alu                'Do the bit set
                        rdword  address, h_reg             'Put byte to memory (HL)
                        call    #write_memory_byte
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'SET [IXY + disp]
set_idx_lmm
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

#endif

'---------------------------------------------------------------------------------------------------------
'Break LMM.
break_lmm               wrword  pc, pc_reg
                        wrbyte  flags, f_reg
                        wrword  sp, sp_reg                         
                        mov     data_8, #io_cmd_break      'Set I/O command to BREAK
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'Pass I/O to another cog
#endif
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz
              if_nz     sub     lmm_pc, #2*4               'JMP #:wait  in LMM
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'Take I/O from another cog
#endif
                        jmp     run_break_lmm_ret          '
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
dispatch_table_00
{00}                    long    fetch                                                                     'NOP
{01}                    long    get_immediate_word + (put_bc              << 9)                           'LXI  B  *
{02}                    long    get_a              + (put_memory_bc       << 9)                           'STAX B  *
{03}                    long    get_bc             + (increment_word      << 9) + (put_bc          << 18) 'INX  B  *
{04}                    long    get_b              + (increment_byte      << 9) + (put_b           << 18) 'INR  B  *
{05}                    long    get_b              + (decrement_byte      << 9) + (put_b           << 18) 'DCR  B  *
{06}                    long    get_immediate_byte + (put_b               << 9)                           'MVI  B  *
{07}                    long    get_a              + (alu_rlc             << 9) + (put_a           << 18) 'RLC     *
{08}                    long    not_implemented                                                           'EX      AF,AF'
{09}                    long    get_bc             + (double_add_hl       << 9) + (put_hl          << 18) 'DAD  B  *
{0A}                    long    get_memory_bc      + (put_a               << 9)                           'LDAX B  *
{0B}                    long    get_bc             + (decrement_word      << 9) + (put_bc          << 18) 'DCX  B  *
{0C}                    long    get_c              + (increment_byte      << 9) + (put_c           << 18) 'INR  C  *
{0D}                    long    get_c              + (decrement_byte      << 9) + (put_c           << 18) 'DCR  C  *
{0E}                    long    get_immediate_byte + (put_c               << 9)                           'MVI  C  *
{0F}                    long    get_a              + (alu_rrc             << 9) + (put_a           << 18) 'RRC     *

{10}                    long    dec_jmp_nz                                                                'DJNZ disp     
{11}                    long    get_immediate_word + (put_de              << 9)                           'LXI  D  *
{12}                    long    get_a              + (put_memory_de       << 9)                           'STAX D  *
{13}                    long    get_de             + (increment_word      << 9) + (put_de          << 18) 'INX  D  *
{14}                    long    get_d              + (increment_byte      << 9) + (put_d           << 18) 'INR  D  *
{15}                    long    get_d              + (decrement_byte      << 9) + (put_d           << 18) 'DCR  D  *
{16}                    long    get_immediate_byte + (put_d               << 9)                           'MVI  D  *
{17}                    long    get_a              + (alu_ral             << 9) + (put_a           << 18) 'RAL     *
{18}                    long    jump_rel_unconditional                                                    'JR   disp
{19}                    long    get_de             + (double_add_hl       << 9) + (put_hl          << 18) 'DAD  D  *
{1A}                    long    get_memory_de      + (put_a               << 9)                           'LDAX D  *
{1B}                    long    get_de             + (decrement_word      << 9) + (put_de          << 18) 'DCX  D  *
{1C}                    long    get_e              + (increment_byte      << 9) + (put_e           << 18) 'INR  E  *
{1D}                    long    get_e              + (decrement_byte      << 9) + (put_e           << 18) 'DCR  E  *
{1E}                    long    get_immediate_byte + (put_e               << 9)                           'MVI  E  *
{1F}                    long    get_a              + (alu_rar             << 9) + (put_a           << 18) 'RAR     *

{20}                    long    test_flag_false    + (jump_rel_conditional<< 9) + (zero_bit        << 18) 'JR   NZ,disp
{21}                    long    get_immediate_word + (put_hl              << 9)                           'LXI  H  *
{22}                    long    get_hl             + (store_word_direct   << 9)                           'SHLD    *
{23}                    long    get_hl             + (increment_word      << 9) + (put_hl          << 18) 'INX  H  *
{24}                    long    get_h              + (increment_byte      << 9) + (put_h           << 18) 'INR  H  *
{25}                    long    get_h              + (decrement_byte      << 9) + (put_h           << 18) 'DCR  H  *
{26}                    long    get_immediate_byte + (put_h               << 9)                           'MVI  H  *
{27}                    long    vector_lmm         + (daa_lmm_no          << 9)                           'DAA
{28}                    long    test_flag_true     + (jump_rel_conditional<< 9) + (zero_bit        << 18) 'JR   Z,disp
{29}                    long    get_hl             + (double_add_hl       << 9) + (put_hl          << 18) 'DAD  H  *
{2A}                    long    load_word_direct   + (put_hl              << 9)                           'LHLD    *
{2B}                    long    get_hl             + (decrement_word      << 9) + (put_hl          << 18) 'DCX  H  *
{2C}                    long    get_l              + (increment_byte      << 9) + (put_l           << 18) 'INR  L  *
{2D}                    long    get_l              + (decrement_byte      << 9) + (put_l           << 18) 'DCR  L  *
{2E}                    long    get_immediate_byte + (put_l               << 9)                           'MVI  L  *
{2F}                    long    get_a              + (complement_byte     << 9) + (put_a           << 18) 'CMA     *

{30}                    long    test_flag_false    + (jump_rel_conditional<< 9) + (carry_bit       << 18) 'JR   NC,disp
{31}                    long    get_immediate_word + (put_sp              << 9)                           'LXI  SP *
{32}                    long    get_immediate_word + (store_a_direct      << 9)                           'STA     *
{33}                    long    get_sp             + (increment_word      << 9) + (put_sp          << 18) 'INX  SP *
{34}                    long    get_memory_hl      + (increment_byte      << 9) + (put_memory_hl   << 18) 'INR  M  *
{35}                    long    get_memory_hl      + (decrement_byte      << 9) + (put_memory_hl   << 18) 'DCR  M  *
{36}                    long    get_immediate_byte + (put_memory_hl       << 9)                           'MVI  M  *
{37}                    long    set_carry                                                                 'STC     *
{38}                    long    test_flag_true     + (jump_rel_conditional<< 9) + (carry_bit       << 18) 'JR   C,disp
{39}                    long    get_sp             + (double_add_hl       << 9) + (put_hl          << 18) 'DAD  SP *
{3A}                    long    get_immediate_word + (load_a_direct       << 9)                           'LDA     *
{3B}                    long    get_sp             + (decrement_word      << 9) + (put_sp          << 18) 'DCX  SP *
{3C}                    long    get_a              + (increment_byte      << 9) + (put_a           << 18) 'INR  A  *
{3D}                    long    get_a              + (decrement_byte      << 9) + (put_a           << 18) 'DCR  A  *
{3E}                    long    get_immediate_byte + (put_a               << 9)                           'MVI  A  *
{3F}                    long    complement_carry                                                          'CMC     *


{40}                    long    fetch                                                                     'MOV B,B
{41}                    long    get_c              + (put_b               << 9)                           'MOV B,C
{42}                    long    get_d              + (put_b               << 9)                           'MOV B,D
{43}                    long    get_e              + (put_b               << 9)                           'MOV B,E
{44}                    long    get_h              + (put_b               << 9)                           'MOV B,H
{45}                    long    get_l              + (put_b               << 9)                           'MOV B,L
{46}                    long    get_memory_hl      + (put_b               << 9)                           'MOV B,M
{47}                    long    get_a              + (put_b               << 9)                           'MOV B,A
{48}                    long    get_b              + (put_c               << 9)                           'MOV C,B
{49}                    long    fetch                                                                     'MOV C,C
{4A}                    long    get_d              + (put_c               << 9)                           'MOV C,D
{4B}                    long    get_e              + (put_c               << 9)                           'MOV C,E
{4C}                    long    get_h              + (put_c               << 9)                           'MOV C,H
{4D}                    long    get_l              + (put_c               << 9)                           'MOV C,L
{4E}                    long    get_memory_hl      + (put_c               << 9)                           'MOV C,M
{4F}                    long    get_a              + (put_c               << 9)                           'MOV C,A

{50}                    long    get_b              + (put_d               << 9)                           'MOV D,B
{51}                    long    get_c              + (put_d               << 9)                           'MOV D,C
{52}                    long    fetch                                                                     'MOV D,D
{53}                    long    get_e              + (put_d               << 9)                           'MOV D,E
{54}                    long    get_h              + (put_d               << 9)                           'MOV D,H
{55}                    long    get_l              + (put_d               << 9)                           'MOV D,L
{56}                    long    get_memory_hl      + (put_d               << 9)                           'MOV D,M
{57}                    long    get_a              + (put_d               << 9)                           'MOV D,A
{58}                    long    get_b              + (put_e               << 9)                           'MOV E,B
{59}                    long    get_c              + (put_e               << 9)                           'MOV E,C
{5A}                    long    get_d              + (put_e               << 9)                           'MOV E,D
{5B}                    long    fetch                                                                     'MOV E,E
{5C}                    long    get_h              + (put_e               << 9)                           'MOV E,H
{5D}                    long    get_l              + (put_e               << 9)                           'MOV E,L
{5E}                    long    get_memory_hl      + (put_e               << 9)                           'MOV E,M
{5F}                    long    get_a              + (put_e               << 9)                           'MOV E,A

{60}                    long    get_b              + (put_h               << 9)                           'MOV H,B
{61}                    long    get_c              + (put_h               << 9)                           'MOV H,C
{62}                    long    get_d              + (put_h               << 9)                           'MOV H,D
{63}                    long    get_e              + (put_h               << 9)                           'MOV H,E
{64}                    long    fetch                                                                     'MOV H,H
{65}                    long    get_l              + (put_h               << 9)                           'MOV H,L
{66}                    long    get_memory_hl      + (put_h               << 9)                           'MOV H,M
{67}                    long    get_a              + (put_h               << 9)                           'MOV H,A
{68}                    long    get_b              + (put_l               << 9)                           'MOV L,B
{69}                    long    get_c              + (put_l               << 9)                           'MOV L,C
{6A}                    long    get_d              + (put_l               << 9)                           'MOV L,D
{6B}                    long    get_e              + (put_l               << 9)                           'MOV L,E
{6C}                    long    get_h              + (put_l               << 9)                           'MOV L,H
{6D}                    long    fetch                                                                     'MOV L,L
{6E}                    long    get_memory_hl      + (put_l               << 9)                           'MOV L,M
{6F}                    long    get_a              + (put_l               << 9)                           'MOV L,A

{70}                    long    get_b              + (put_memory_hl       << 9)                           'MOV M,B
{71}                    long    get_c              + (put_memory_hl       << 9)                           'MOV M,C
{72}                    long    get_d              + (put_memory_hl       << 9)                           'MOV M,D
{73}                    long    get_e              + (put_memory_hl       << 9)                           'MOV M,E
{74}                    long    get_h              + (put_memory_hl       << 9)                           'MOV M,H
{75}                    long    get_l              + (put_memory_hl       << 9)                           'MOV M,L
{76}                    long    halt                                                                      'HALT
{77}                    long    get_a              + (put_memory_hl       << 9)                           'MOV M,A
{78}                    long    get_b              + (put_a               << 9)                           'MOV A,B
{79}                    long    get_c              + (put_a               << 9)                           'MOV A,C
{7A}                    long    get_d              + (put_a               << 9)                           'MOV A,D
{7B}                    long    get_e              + (put_a               << 9)                           'MOV A,E
{7C}                    long    get_h              + (put_a               << 9)                           'MOV A,H
{7D}                    long    get_l              + (put_a               << 9)                           'MOV A,L
{7E}                    long    get_memory_hl      + (put_a               << 9)                           'MOV A,M
{7F}                    long    fetch                                                                     'MOV A,A


{80}                    long    get_b              + (alu_add             << 9) + (put_a           << 18) 'ADD B *
{81}                    long    get_c              + (alu_add             << 9) + (put_a           << 18) 'ADD C *
{82}                    long    get_d              + (alu_add             << 9) + (put_a           << 18) 'ADD D *
{83}                    long    get_e              + (alu_add             << 9) + (put_a           << 18) 'ADD E *
{84}                    long    get_h              + (alu_add             << 9) + (put_a           << 18) 'ADD H *
{85}                    long    get_l              + (alu_add             << 9) + (put_a           << 18) 'ADD L *
{86}                    long    get_memory_hl      + (alu_add             << 9) + (put_a           << 18) 'ADD M *
{87}                    long    get_a              + (alu_add             << 9) + (put_a           << 18) 'ADD A *
{88}                    long    get_b              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC B *
{89}                    long    get_c              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC C *
{8A}                    long    get_d              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC D *
{8B}                    long    get_e              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC E *
{8C}                    long    get_h              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC H *
{8D}                    long    get_l              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC L *
{8E}                    long    get_memory_hl      + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC M *
{8F}                    long    get_a              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC A *

{90}                    long    get_b              + (alu_sub             << 9) + (put_a           << 18) 'SUB B * 
{91}                    long    get_c              + (alu_sub             << 9) + (put_a           << 18) 'SUB C *
{92}                    long    get_d              + (alu_sub             << 9) + (put_a           << 18) 'SUB D *
{93}                    long    get_e              + (alu_sub             << 9) + (put_a           << 18) 'SUB E *
{94}                    long    get_h              + (alu_sub             << 9) + (put_a           << 18) 'SUB H *
{95}                    long    get_l              + (alu_sub             << 9) + (put_a           << 18) 'SUB L *
{96}                    long    get_memory_hl      + (alu_sub             << 9) + (put_a           << 18) 'SUB M *
{97}                    long    get_a              + (alu_sub             << 9) + (put_a           << 18) 'SUB A *
{98}                    long    get_b              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB B *
{99}                    long    get_c              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB C *
{9A}                    long    get_d              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB D *
{9B}                    long    get_e              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB E *
{9C}                    long    get_h              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB H *
{9D}                    long    get_l              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB L *
{9E}                    long    get_memory_hl      + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB M *
{9F}                    long    get_a              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB A *

{A0}                    long    get_b              + (alu_and             << 9) + (put_a           << 18) 'AND B * 
{A1}                    long    get_c              + (alu_and             << 9) + (put_a           << 18) 'AND C *
{A2}                    long    get_d              + (alu_and             << 9) + (put_a           << 18) 'AND D *
{A3}                    long    get_e              + (alu_and             << 9) + (put_a           << 18) 'AND E *
{A4}                    long    get_h              + (alu_and             << 9) + (put_a           << 18) 'AND H *
{A5}                    long    get_l              + (alu_and             << 9) + (put_a           << 18) 'AND L *
{A6}                    long    get_memory_hl      + (alu_and             << 9) + (put_a           << 18) 'AND M *
{A7}                    long    get_a              + (alu_and             << 9) + (put_a           << 18) 'AND A *
{A8}                    long    get_b              + (alu_xor             << 9) + (put_a           << 18) 'XRA B *
{A9}                    long    get_c              + (alu_xor             << 9) + (put_a           << 18) 'XRA C *
{AA}                    long    get_d              + (alu_xor             << 9) + (put_a           << 18) 'XRA D *
{AB}                    long    get_e              + (alu_xor             << 9) + (put_a           << 18) 'XRA E *
{AC}                    long    get_h              + (alu_xor             << 9) + (put_a           << 18) 'XRA H *
{AD}                    long    get_l              + (alu_xor             << 9) + (put_a           << 18) 'XRA L *
{AE}                    long    get_memory_hl      + (alu_xor             << 9) + (put_a           << 18) 'XRA M *
{AF}                    long    get_a              + (alu_xor             << 9) + (put_a           << 18) 'XRA A *

{B0}                    long    get_b              + (alu_or              << 9) + (put_a           << 18) 'ORA B *
{B1}                    long    get_c              + (alu_or              << 9) + (put_a           << 18) 'ORA C *
{B2}                    long    get_d              + (alu_or              << 9) + (put_a           << 18) 'ORA D *
{B3}                    long    get_e              + (alu_or              << 9) + (put_a           << 18) 'ORA E *
{B4}                    long    get_h              + (alu_or              << 9) + (put_a           << 18) 'ORA H *
{B5}                    long    get_l              + (alu_or              << 9) + (put_a           << 18) 'ORA L *
{B6}                    long    get_memory_hl      + (alu_or              << 9) + (put_a           << 18) 'ORA M *
{B7}                    long    get_a              + (alu_or              << 9) + (put_a           << 18) 'ORA A *
{B8}                    long    get_b              + (alu_sub             << 9) + (fetch           << 18) 'CMP B * FIXME cmp has no result !!!
{B9}                    long    get_c              + (alu_sub             << 9) + (fetch           << 18) 'CMP C *
{BA}                    long    get_d              + (alu_sub             << 9) + (fetch           << 18) 'CMP D *
{BB}                    long    get_e              + (alu_sub             << 9) + (fetch           << 18) 'CMP E *
{BC}                    long    get_h              + (alu_sub             << 9) + (fetch           << 18) 'CMP H *
{BD}                    long    get_l              + (alu_sub             << 9) + (fetch           << 18) 'CMP L *
{BE}                    long    get_memory_hl      + (alu_sub             << 9) + (fetch           << 18) 'CMP M *
{BF}                    long    get_a              + (alu_sub             << 9) + (fetch           << 18) 'CMP A *

{C0}                    long    test_flag_false    + (return_conditional  << 9) + (zero_bit        << 18) '*RNZ 
{C1}                    long    pop_               + (put_bc              << 9)                           '* POP B
{C2}                    long    test_flag_false    + (jump_conditional    << 9) + (zero_bit        << 18) '* JNZ 
{C3}                    long    jump_unconditional                                                        '* JMP 
{C4}                    long    test_flag_false    + (call_conditional    << 9) + (zero_bit        << 18) '*CNZ 
{C5}                    long    get_bc             + (push_               << 9)                           '*PUSH B
{C6}                    long    get_immediate_byte + (alu_add             << 9) + (put_a           << 18) '*ADI 
{C7}                    long    restart            + ((0 << 3)            << 9)                           '*RST 0
{C8}                    long    test_flag_true     + (return_conditional  << 9) + (zero_bit        << 18) '* RZ 
{C9}                    long    return_unconditional                                                      '*RET 
{CA}                    long    test_flag_true     + (jump_conditional    << 9) + (zero_bit        << 18) '*JZ 
#ifdef CPU_Z80
{CB}                    long    z80_CB                                                                    '*Z80, CB Prefix
#else
{CB}                    long    undocumented
#endif
{CC}                    long    test_flag_true     + (call_conditional    << 9) + (zero_bit        << 18) '*CZ
{CD}                    long    call_unconditional                                                        '*CALL 
{CE}                    long    get_immediate_byte + (alu_add_with_carry  << 9) + (put_a           << 18) '*ACI 
{CF}                    long    restart            + ((1 << 3)            << 9)                           '*RST 1

{D0}                    long    test_flag_false    + (return_conditional  << 9) + (carry_bit       << 18) '*RNC 
{D1}                    long    pop_               + (put_de              << 9)                           '*POP D
{D2}                    long    test_flag_false    + (jump_conditional    << 9) + (carry_bit       << 18) '*JNC 
{D3}                    long    vector_lmm         + (out_lmm_no          << 9)                           '*OUT
{D4}                    long    test_flag_false    + (call_conditional    << 9) + (carry_bit       << 18) '*CNC
{D5}                    long    get_de             + (push_               << 9)                           '*PUSH D
{D6}                    long    get_immediate_byte + (alu_sub             << 9) + (put_a           << 18) '*SUI
{D7}                    long    restart            + ((2 << 3)            << 9)                           '*RST 2
{D8}                    long    test_flag_true     + (return_conditional  << 9) + (carry_bit       << 18) '*RC 
{D9}                    long    z80_exx                                                                   '*EXX, D9, BC/DE/HL <-> BC'/DE'/HL' 
{DA}                    long    test_flag_true     + (jump_conditional    << 9) + (carry_bit       << 18) '*JC 
{DB}                    long    vector_lmm         + (in_lmm_no           << 9)                           'IN
{DC}                    long    test_flag_true     + (call_conditional    << 9) + (carry_bit       << 18) '*CC 
#ifdef CPU_Z80
{DD}                    long    z80_DD                                                                    '*Z80 DD Prefix
#else
{DD}                    long    undocumented
#endif
{DE}                    long    get_immediate_byte + (alu_sub_with_borrow << 9) + (put_a           << 18) '*SBI
{DF}                    long    restart            + ((3 << 3)            << 9)                           '*RST 3

{E0}                    long    test_flag_false    + (return_conditional  << 9) + (parity_bit      << 18) '*RPO 
{E1}                    long    pop_               + (put_hl              << 9)                           '*POP H
{E2}                    long    test_flag_false    + (jump_conditional    << 9) + (parity_bit      << 18) '*JPO 
{E3}                    long    get_tos            + (exchange            << 9) + (put_tos         << 18) '*XTHL,    H <-> (SP+1); L <-> (SP) 
{E4}                    long    test_flag_false    + (call_conditional    << 9) + (parity_bit      << 18) '*CPO 
{E5}                    long    get_hl             + (push_               << 9)                           '*PUSH H
{E6}                    long    get_immediate_byte + (alu_and             << 9) + (put_a           << 18) '*ANI 
{E7}                    long    restart            + ((4 << 3)            << 9)                           '*RST 4
{E8}                    long    test_flag_true     + (return_conditional  << 9) + (parity_bit      << 18) '*RPE 
{E9}                    long    get_hl             + (put_pc              << 9)                           '*PCHL,    PC <- HL
{EA}                    long    test_flag_true     + (jump_conditional    << 9) + (parity_bit      << 18) '*JPE 
{EB}                    long    get_de             + (exchange            << 9) + (put_de          << 18) '*XCHG,    HL <-> DE  
{EC}                    long    test_flag_true     + (call_conditional    << 9) + (parity_bit      << 18) '*CPE 
#ifdef CPU_Z80
{ED}                    long    z80_ED                                                                    '*Z80 ED Prefix
#else
{ED}                    long    undocumented
#endif
{EE}                    long    get_immediate_byte + (alu_xor             << 9) + (put_a           << 18) '*XRI
{EF}                    long    restart            + ((5 << 3)            << 9)                           '*RST 5

{F0}                    long    test_flag_false    + (return_conditional  << 9) + (sign_bit        << 18) '*RP 
{F1}                    long    pop_               + (put_af              << 9)                           '*POP AF
{F2}                    long    test_flag_false    + (jump_conditional    << 9) + (sign_bit        << 18) '*JP 
{F3}                    long    disable_interrupts                                                        '*DI,  Disable interrupts
{F4}                    long    test_flag_false    + (call_conditional    << 9) + (sign_bit        << 18) '*CP 
{F5}                    long    get_af             + (push_               << 9)                           '*PUSH AF
{F6}                    long    get_immediate_byte + (alu_or              << 9) + (put_a           << 18) '*ORI 
{F7}                    long    restart            + ((6 << 3)            << 9)                           '*RST 6
{F8}                    long    test_flag_true     + (return_conditional  << 9) + (sign_bit        << 18) '*RM 
{F9}                    long    get_hl             + (put_sp              << 9)                           '*SPHL,    SP <- HL 
{FA}                    long    test_flag_true     + (jump_conditional    << 9) + (sign_bit        << 18) '*JM 
{FB}                    long    enable_interrupts                                                         '*EI, Enable interrupts
{FC}                    long    test_flag_true     + (call_conditional    << 9) + (sign_bit        << 18) '*CM 
#ifdef CPU_Z80
{FD}                    long    z80_FD                                                                    '*Z80 FD Prefix
#else
{FD}                    long    undocumented
#endif
{FE}                    long    get_immediate_byte + (alu_sub             << 9) + (fetch           << 18) '*CPI
{FF}                    long    restart            + ((7 << 3)            << 9)                           '*RST 7
'---------------------------------------------------------------------------------------------------------

#ifdef CPU_Z80
'---------------------------------------------------------------------------------------------------------
dispatch_table_cb
{00}                    long    get_b              + (alu_rlc             << 9) + (put_b           << 18) 'RLC     B               CB00            ---
{01}                    long    get_c              + (alu_rlc             << 9) + (put_c           << 18) 'RLC     C               CB01            ---
{02}                    long    get_d              + (alu_rlc             << 9) + (put_d           << 18) 'RLC     D               CB02            ---
{03}                    long    get_e              + (alu_rlc             << 9) + (put_e           << 18) 'RLC     E               CB03            ---
{04}                    long    get_h              + (alu_rlc             << 9) + (put_h           << 18) 'RLC     H               CB04            ---
{05}                    long    get_l              + (alu_rlc             << 9) + (put_l           << 18) 'RLC     L               CB05            ---
{06}                    long    get_memory_hl      + (alu_rlc             << 9) + (put_memory_hl   << 18) 'RLC     (HL)            CB06            ---
{07}                    long    get_a              + (alu_rlc             << 9) + (put_a           << 18) 'RLC     A               CB07            ---
{08}                    long    get_b              + (alu_rrc             << 9) + (put_b           << 18) 'RRC     B               CB08            ---
{09}                    long    get_c              + (alu_rrc             << 9) + (put_c           << 18) 'RRC     C               CB09            ---
{0A}                    long    get_d              + (alu_rrc             << 9) + (put_d           << 18) 'RRC     D               CB0A            ---
{0B}                    long    get_e              + (alu_rrc             << 9) + (put_e           << 18) 'RRC     E               CB0B            ---
{0C}                    long    get_h              + (alu_rrc             << 9) + (put_h           << 18) 'RRC     H               CB0C            ---
{0D}                    long    get_l              + (alu_rrc             << 9) + (put_l           << 18) 'RRC     L               CB0D            ---
{0E}                    long    get_memory_hl      + (alu_rrc             << 9) + (put_memory_hl   << 18) 'RRC     (HL)            CB0E            ---
{0F}                    long    get_a              + (alu_rrc             << 9) + (put_a           << 18) 'RRC     A               CB0F            ---

{10}                    long    get_b              + (alu_ral             << 9) + (put_b           << 18) 'RL      B               CB10            ---
{11}                    long    get_c              + (alu_ral             << 9) + (put_c           << 18) 'RL      C               CB11            ---
{12}                    long    get_d              + (alu_ral             << 9) + (put_d           << 18) 'RL      D               CB12            ---
{13}                    long    get_e              + (alu_ral             << 9) + (put_e           << 18) 'RL      E               CB13            ---
{14}                    long    get_h              + (alu_ral             << 9) + (put_h           << 18) 'RL      H               CB14            ---
{15}                    long    get_l              + (alu_ral             << 9) + (put_l           << 18) 'RL      L               CB15            ---
{16}                    long    get_memory_hl      + (alu_ral             << 9) + (put_memory_hl   << 18) 'RL      (HL)            CB16            ---
{17}                    long    get_a              + (alu_ral             << 9) + (put_a           << 18) 'RL      A               CB17            ---
{18}                    long    get_b              + (alu_rar             << 9) + (put_b           << 18) 'RR      B               CB18            ---
{19}                    long    get_c              + (alu_rar             << 9) + (put_c           << 18) 'RR      C               CB19            ---
{1A}                    long    get_d              + (alu_rar             << 9) + (put_d           << 18) 'RR      D               CB1A            ---
{1B}                    long    get_e              + (alu_rar             << 9) + (put_e           << 18) 'RR      E               CB1B            ---
{1C}                    long    get_h              + (alu_rar             << 9) + (put_h           << 18) 'RR      H               CB1C            ---
{1D}                    long    get_l              + (alu_rar             << 9) + (put_l           << 18) 'RR      L               CB1D            ---
{1E}                    long    get_memory_hl      + (alu_rar             << 9) + (put_memory_hl   << 18) 'RR      (HL)            CB1E            ---
{1F}                    long    get_a              + (alu_rar             << 9) + (put_a           << 18) 'RR      A               CB1F            ---

{20}                    long    get_b              + (alu_sla             << 9) + (put_b           << 18) 'SLA     B               CB20            ---
{21}                    long    get_c              + (alu_sla             << 9) + (put_c           << 18) 'SLA     C               CB21            ---
{22}                    long    get_d              + (alu_sla             << 9) + (put_d           << 18) 'SLA     D               CB22            ---
{23}                    long    get_e              + (alu_sla             << 9) + (put_e           << 18) 'SLA     E               CB23            ---
{24}                    long    get_h              + (alu_sla             << 9) + (put_h           << 18) 'SLA     H               CB24            ---
{25}                    long    get_l              + (alu_sla             << 9) + (put_l           << 18) 'SLA     L               CB25            ---
{26}                    long    get_memory_hl      + (alu_sla             << 9) + (put_memory_hl   << 18) 'SLA     (HL)            CB26            ---
{27}                    long    get_a              + (alu_sla             << 9) + (put_a           << 18) 'SLA     A               CB27            ---
{28}                    long    get_b              + (alu_sra             << 9) + (put_b           << 18) 'SRA     B               CB28            ---
{29}                    long    get_c              + (alu_sra             << 9) + (put_c           << 18) 'SRA     C               CB29            ---
{2A}                    long    get_d              + (alu_sra             << 9) + (put_d           << 18) 'SRA     D               CB2A            ---
{2B}                    long    get_e              + (alu_sra             << 9) + (put_e           << 18) 'SRA     E               CB2B            ---
{2C}                    long    get_h              + (alu_sra             << 9) + (put_h           << 18) 'SRA     H               CB2C            ---
{2D}                    long    get_l              + (alu_sra             << 9) + (put_l           << 18) 'SRA     L               CB2D            ---
{2E}                    long    get_memory_hl      + (alu_sra             << 9) + (put_memory_hl   << 18) 'SRA     (HL)            CB2E            ---
{2F}                    long    get_a              + (alu_sra             << 9) + (put_a           << 18) 'SRA     A               CB2F            ---

{30}                    long    undocumented
{31}                    long    undocumented
{32}                    long    undocumented
{33}                    long    undocumented
{34}                    long    undocumented
{35}                    long    undocumented
{36}                    long    undocumented
{37}                    long    undocumented
{38}                    long    get_b              + (alu_srl             << 9) + (put_b           << 18) 'SRL     B               CB38            ---
{39}                    long    get_c              + (alu_srl             << 9) + (put_c           << 18) 'SRL     C               CB39            ---
{3A}                    long    get_d              + (alu_srl             << 9) + (put_d           << 18) 'SRL     D               CB3A            ---
{3B}                    long    get_e              + (alu_srl             << 9) + (put_e           << 18) 'SRL     E               CB3B            ---
{3C}                    long    get_h              + (alu_srl             << 9) + (put_h           << 18) 'SRL     H               CB3C            ---
{3D}                    long    get_l              + (alu_srl             << 9) + (put_l           << 18) 'SRL     L               CB3D            ---
{3E}                    long    get_memory_hl      + (alu_srl             << 9) + (put_memory_hl   << 18) 'SRL     (HL)            CB3E            ---
{3F}                    long    get_a              + (alu_srl             << 9) + (put_a           << 18) 'SRL     A               CB3F            ---


{40}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (b_reg << 23) 'BIT     0,B             CB40            Z flag <- NOT 0b
{41}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (c_reg << 23) 'BIT     0,C             CB41            Z flag <- NOT 0b
{42}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (d_reg << 23) 'BIT     0,D             CB42            Z flag <- NOT 0b
{43}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (e_reg << 23) 'BIT     0,E             CB43            Z flag <- NOT 0b
{44}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (h_reg << 23) 'BIT     0,H             CB44            Z flag <- NOT 0b
{45}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (l_reg << 23) 'BIT     0,L             CB45            Z flag <- NOT 0b
{46}                    long    vector_lmm     + (bit_mem_lmm_no      << 9) + (bit_0_mask      << 15)                 'BIT     0,(HL)          CB46            Z flag <- NOT 0b
{47}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (a_reg << 23) 'BIT     0,A             CB47            Z flag <- NOT 0b
{48}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (b_reg << 23) 'BIT     1,B             CB48            Z flag <- NOT 1b
{49}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (c_reg << 23) 'BIT     1,C             CB49            Z flag <- NOT 1b
{4A}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (d_reg << 23) 'BIT     1,D             CB4A            Z flag <- NOT 1b
{4B}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (e_reg << 23) 'BIT     1,E             CB4B            Z flag <- NOT 1b
{4C}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (h_reg << 23) 'BIT     1,H             CB4C            Z flag <- NOT 1b
{4D}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (l_reg << 23) 'BIT     1,L             CB4D            Z flag <- NOT 1b
{4E}                    long    vector_lmm     + (bit_mem_lmm_no      << 9) + (bit_1_mask      << 15)                 'BIT     1,(HL)          CB4E            Z flag <- NOT 1b
{4F}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (a_reg << 23) 'BIT     1,A             CB4F            Z flag <- NOT 1b

{50}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (b_reg << 23) 'BIT     2,B             CB50            Z flag <- NOT 2b
{51}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (c_reg << 23) 'BIT     2,C             CB51            Z flag <- NOT 2b
{52}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (d_reg << 23) 'BIT     2,D             CB52            Z flag <- NOT 2b
{53}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (e_reg << 23) 'BIT     2,E             CB53            Z flag <- NOT 2b
{54}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (h_reg << 23) 'BIT     2,H             CB54            Z flag <- NOT 2b
{55}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (l_reg << 23) 'BIT     2,L             CB55            Z flag <- NOT 2b
{56}                    long    vector_lmm     + (bit_mem_lmm_no      << 9) + (bit_2_mask      << 15)                 'BIT     2,(HL)          CB56            Z flag <- NOT 2b
{57}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (a_reg << 23) 'BIT     2,A             CB57            Z flag <- NOT 2b
{58}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (b_reg << 23) 'BIT     3,B             CB58            Z flag <- NOT 3b
{59}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (c_reg << 23) 'BIT     3,C             CB59            Z flag <- NOT 3b
{5A}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (d_reg << 23) 'BIT     3,D             CB5A            Z flag <- NOT 3b
{5B}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (e_reg << 23) 'BIT     3,E             CB5B            Z flag <- NOT 3b
{5C}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (h_reg << 23) 'BIT     3,H             CB5C            Z flag <- NOT 3b
{5D}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (l_reg << 23) 'BIT     3,L             CB5D            Z flag <- NOT 3b
{5E}                    long    vector_lmm     + (bit_mem_lmm_no      << 9) + (bit_3_mask      << 15)                 'BIT     3,(HL)          CB5E            Z flag <- NOT 3b
{5F}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (a_reg << 23) 'BIT     3,A             CB5F            Z flag <- NOT 3b

{60}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (b_reg << 23) 'BIT     4,B             CB60            Z flag <- NOT 4b
{61}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (c_reg << 23) 'BIT     4,C             CB61            Z flag <- NOT 4b
{62}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (d_reg << 23) 'BIT     4,D             CB62            Z flag <- NOT 4b
{63}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (e_reg << 23) 'BIT     4,E             CB63            Z flag <- NOT 4b
{64}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (h_reg << 23) 'BIT     4,H             CB64            Z flag <- NOT 4b
{65}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (l_reg << 23) 'BIT     4,L             CB65            Z flag <- NOT 4b
{66}                    long    vector_lmm     + (bit_mem_lmm_no      << 9) + (bit_4_mask      << 15)                 'BIT     4,(HL)          CB66            Z flag <- NOT 4b
{67}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (a_reg << 23) 'BIT     4,A             CB67            Z flag <- NOT 4b
{68}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (b_reg << 23) 'BIT     5,B             CB68            Z flag <- NOT 5b
{69}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (c_reg << 23) 'BIT     5,C             CB69            Z flag <- NOT 5b
{6A}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (d_reg << 23) 'BIT     5,D             CB6A            Z flag <- NOT 5b
{6B}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (e_reg << 23) 'BIT     5,E             CB6B            Z flag <- NOT 5b
{6C}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (h_reg << 23) 'BIT     5,H             CB6C            Z flag <- NOT 5b
{6D}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (l_reg << 23) 'BIT     5,L             CB6D            Z flag <- NOT 5b
{6E}                    long    vector_lmm     + (bit_mem_lmm_no      << 9) + (bit_5_mask      << 15)                 'BIT     5,(HL)          CB6E            Z flag <- NOT 5b
{6F}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (a_reg << 23) 'BIT     5,A             CB6F            Z flag <- NOT 5b

{70}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (b_reg << 23) 'BIT     6,B             CB70            Z flag <- NOT 6b
{71}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (c_reg << 23) 'BIT     6,C             CB71            Z flag <- NOT 6b
{72}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (d_reg << 23) 'BIT     6,D             CB72            Z flag <- NOT 6b
{73}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (e_reg << 23) 'BIT     6,E             CB73            Z flag <- NOT 6b
{74}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (h_reg << 23) 'BIT     6,H             CB74            Z flag <- NOT 6b
{75}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (l_reg << 23) 'BIT     6,L             CB75            Z flag <- NOT 6b
{76}                    long    vector_lmm     + (bit_mem_lmm_no      << 9) + (bit_6_mask      << 15)                 'BIT     6,(HL)          CB76            Z flag <- NOT 6b
{77}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (a_reg << 23) 'BIT     6,A             CB77            Z flag <- NOT 6b
{78}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (b_reg << 23) 'BIT     7,B             CB78            Z flag <- NOT 7b
{79}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (c_reg << 23) 'BIT     7,C             CB79            Z flag <- NOT 7b
{7A}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (d_reg << 23) 'BIT     7,D             CB7A            Z flag <- NOT 7b
{7B}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (e_reg << 23) 'BIT     7,E             CB7B            Z flag <- NOT 7b
{7C}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (h_reg << 23) 'BIT     7,H             CB7C            Z flag <- NOT 7b
{7D}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (l_reg << 23) 'BIT     7,L             CB7D            Z flag <- NOT 7b
{7E}                    long    vector_lmm     + (bit_mem_lmm_no      << 9) + (bit_7_mask      << 15)                 'BIT     7,(HL)          CB7E            Z flag <- NOT 7b
{7F}                    long    vector_lmm     + (bit_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (a_reg << 23) 'BIT     7,A             CB7F            Z flag <- NOT 7b

{80}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (b_reg << 23) 'RES     0,B             CB80            0b <- 0
{81}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (c_reg << 23) 'RES     0,C             CB81            0b <- 0
{82}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (d_reg << 23) 'RES     0,D             CB82            0b <- 0
{83}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (e_reg << 23) 'RES     0,E             CB83            0b <- 0
{84}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (h_reg << 23) 'RES     0,H             CB84            0b <- 0
{85}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (l_reg << 23) 'RES     0,L             CB85            0b <- 0
{86}                    long    vector_lmm     + (res_mem_lmm_no      << 9) + (bit_0_mask      << 15)                 'RES     0,(HL)          CB86            0b <- 0
{87}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (a_reg << 23) 'RES     0,A             CB87            0b <- 0
{88}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (b_reg << 23) 'RES     1,B             CB88            1b <- 0
{89}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (c_reg << 23) 'RES     1,C             CB89            1b <- 0
{8A}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (d_reg << 23) 'RES     1,D             CB8A            1b <- 0
{8B}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (e_reg << 23) 'RES     1,E             CB8B            1b <- 0
{8C}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (h_reg << 23) 'RES     1,H             CB8C            1b <- 0
{8D}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (l_reg << 23) 'RES     1,L             CB8D            1b <- 0
{8E}                    long    vector_lmm     + (res_mem_lmm_no      << 9) + (bit_1_mask      << 15)                 'RES     1,(HL)          CB8E            1b <- 0
{8F}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (a_reg << 23) 'RES     1,A             CB8F            1b <- 0

{90}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (b_reg << 23) 'RES     2,B             CB90            2b <- 0
{91}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (c_reg << 23) 'RES     2,C             CB91            2b <- 0
{92}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (d_reg << 23) 'RES     2,D             CB92            2b <- 0
{93}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (e_reg << 23) 'RES     2,E             CB93            2b <- 0
{94}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (h_reg << 23) 'RES     2,H             CB94            2b <- 0
{95}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (l_reg << 23) 'RES     2,L             CB95            2b <- 0
{96}                    long    vector_lmm     + (res_mem_lmm_no      << 9) + (bit_2_mask      << 15)                 'RES     2,(HL)          CB96            2b <- 0
{97}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (a_reg << 23) 'RES     2,A             CB97            2b <- 0
{98}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (b_reg << 23) 'RES     3,B             CB98            3b <- 0
{99}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (c_reg << 23) 'RES     3,C             CB99            3b <- 0
{9A}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (d_reg << 23) 'RES     3,D             CB9A            3b <- 0
{9B}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (e_reg << 23) 'RES     3,E             CB9B            3b <- 0
{9C}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (h_reg << 23) 'RES     3,H             CB9C            3b <- 0
{9D}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (l_reg << 23) 'RES     3,L             CB9D            3b <- 0
{9E}                    long    vector_lmm     + (res_mem_lmm_no      << 9) + (bit_3_mask      << 15)                 'RES     3,(HL)          CB9E            3b <- 0
{9F}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (a_reg << 23) 'RES     3,A             CB9F            3b <- 0

{A0}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (b_reg << 23) 'RES     4,B             CBA0            4b <- 0
{A1}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (c_reg << 23) 'RES     4,C             CBA1            4b <- 0
{A2}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (d_reg << 23) 'RES     4,D             CBA2            4b <- 0
{A3}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (e_reg << 23) 'RES     4,E             CBA3            4b <- 0
{A4}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (h_reg << 23) 'RES     4,H             CBA4            4b <- 0
{A5}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (l_reg << 23) 'RES     4,L             CBA5            4b <- 0
{A6}                    long    vector_lmm     + (res_mem_lmm_no      << 9) + (bit_4_mask      << 15)                 'RES     4,(HL)          CBA6            4b <- 0
{A7}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (a_reg << 23) 'RES     4,A             CBA7            4b <- 0
{A8}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (b_reg << 23) 'RES     5,B             CBA8            5b <- 0
{A9}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (c_reg << 23) 'RES     5,C             CBA9            5b <- 0
{AA}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (d_reg << 23) 'RES     5,D             CBAA            5b <- 0
{AB}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (e_reg << 23) 'RES     5,E             CBAB            5b <- 0
{AC}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (h_reg << 23) 'RES     5,H             CBAC            5b <- 0
{AD}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (l_reg << 23) 'RES     5,L             CBAD            5b <- 0
{AE}                    long    vector_lmm     + (res_mem_lmm_no      << 9) + (bit_5_mask      << 15)                 'RES     5,(HL)          CBAE            5b <- 0
{AF}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (a_reg << 23) 'RES     5,A             CBAF            5b <- 0

{B0}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (b_reg << 23) 'RES     6,B             CBB0            6b <- 0
{B1}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (c_reg << 23) 'RES     6,C             CBB1            6b <- 0
{B2}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (d_reg << 23) 'RES     6,D             CBB2            6b <- 0
{B3}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (e_reg << 23) 'RES     6,E             CBB3            6b <- 0
{B4}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (h_reg << 23) 'RES     6,H             CBB4            6b <- 0
{B5}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (l_reg << 23) 'RES     6,L             CBB5            6b <- 0
{B6}                    long    vector_lmm     + (res_mem_lmm_no      << 9) + (bit_6_mask      << 15)                 'RES     6,(HL)          CBB6            6b <- 0
{B7}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (a_reg << 23) 'RES     6,A             CBB7            6b <- 0
{B8}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (b_reg << 23) 'RES     7,B             CBB8            7b <- 0
{B9}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (c_reg << 23) 'RES     7,C             CBB9            7b <- 0
{BA}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (d_reg << 23) 'RES     7,D             CBBA            7b <- 0
{BB}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (e_reg << 23) 'RES     7,E             CBBB            7b <- 0
{BC}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (h_reg << 23) 'RES     7,H             CBBC            7b <- 0
{BD}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (l_reg << 23) 'RES     7,L             CBBD            7b <- 0
{BE}                    long    vector_lmm     + (res_mem_lmm_no      << 9) + (bit_7_mask      << 15)                 'RES     7,(HL)          CBBE            7b <- 0
{BF}                    long    vector_lmm     + (res_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (a_reg << 23) 'RES     7,A             CBBF            7b <- 0

{C0}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (b_reg << 23) 'SET     0,B             CBC0            0b <- 1
{C1}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (c_reg << 23) 'SET     0,C             CBC1            0b <- 1
{C2}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (d_reg << 23) 'SET     0,D             CBC2            0b <- 1
{C3}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (e_reg << 23) 'SET     0,E             CBC3            0b <- 1
{C4}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (h_reg << 23) 'SET     0,H             CBC4            0b <- 1
{C5}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (l_reg << 23) 'SET     0,L             CBC5            0b <- 1
{C6}                    long    vector_lmm     + (set_mem_lmm_no      << 9) + (bit_0_mask      << 15)                 'SET     0,(HL)          CBC6            0b <- 1
{C7}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_0_mask      << 15) + (a_reg << 23) 'SET     0,A             CBC7            0b <- 1
{C8}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (b_reg << 23) 'SET     1,B             CBC8            1b <- 1
{C9}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (c_reg << 23) 'SET     1,C             CBC9            1b <- 1
{CA}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (d_reg << 23) 'SET     1,D             CBCA            1b <- 1
{CB}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (e_reg << 23) 'SET     1,E             CBCB            1b <- 1
{CC}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (h_reg << 23) 'SET     1,H             CBCC            1b <- 1
{CD}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (l_reg << 23) 'SET     1,L             CBCD            1b <- 1
{CE}                    long    vector_lmm     + (set_mem_lmm_no      << 9) + (bit_1_mask      << 15)                 'SET     1,(HL)          CBCE            1b <- 1
{CF}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_1_mask      << 15) + (a_reg << 23) 'SET     1,A             CBCF            1b <- 1

{D0}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (b_reg << 23) 'SET     2,B             CBD0            2b <- 1
{D1}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (c_reg << 23) 'SET     2,C             CBD1            2b <- 1
{D2}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (d_reg << 23) 'SET     2,D             CBD2            2b <- 1
{D3}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (e_reg << 23) 'SET     2,E             CBD3            2b <- 1
{D4}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (h_reg << 23) 'SET     2,H             CBD4            2b <- 1
{D5}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (l_reg << 23) 'SET     2,L             CBD5            2b <- 1
{D6}                    long    vector_lmm     + (set_mem_lmm_no      << 9) + (bit_2_mask      << 15)                 'SET     2,(HL)          CBD6            2b <- 1
{D7}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_2_mask      << 15) + (a_reg << 23) 'SET     2,A             CBD7            2b <- 1
{D8}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (b_reg << 23) 'SET     3,B             CBD8            3b <- 1
{D9}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (c_reg << 23) 'SET     3,C             CBD9            3b <- 1
{DA}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (d_reg << 23) 'SET     3,D             CBDA            3b <- 1
{DB}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (e_reg << 23) 'SET     3,E             CBDB            3b <- 1
{DC}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (h_reg << 23) 'SET     3,H             CBDC            3b <- 1
{DD}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (l_reg << 23) 'SET     3,L             CBDD            3b <- 1
{DE}                    long    vector_lmm     + (set_mem_lmm_no      << 9) + (bit_3_mask      << 15)                 'SET     3,(HL)          CBDE            3b <- 1
{DF}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_3_mask      << 15) + (a_reg << 23) 'SET     3,A             CBDF            3b <- 1

{E0}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (b_reg << 23) 'SET     4,B             CBE0            4b <- 1
{E1}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (c_reg << 23) 'SET     4,C             CBE1            4b <- 1
{E2}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (d_reg << 23) 'SET     4,D             CBE2            4b <- 1
{E3}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (e_reg << 23) 'SET     4,E             CBE3            4b <- 1
{E4}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (h_reg << 23) 'SET     4,H             CBE4            4b <- 1
{E5}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (l_reg << 23) 'SET     4,L             CBE5            4b <- 1
{E6}                    long    vector_lmm     + (set_mem_lmm_no      << 9) + (bit_4_mask      << 15)                 'SET     4,(HL)          CBE6            4b <- 1
{E7}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_4_mask      << 15) + (a_reg << 23) 'SET     4,A             CBE7            4b <- 1
{E8}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (b_reg << 23) 'SET     5,B             CBE8            5b <- 1
{E9}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (c_reg << 23) 'SET     5,C             CBE9            5b <- 1
{EA}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (d_reg << 23) 'SET     5,D             CBEA            5b <- 1
{EB}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (e_reg << 23) 'SET     5,E             CBEB            5b <- 1
{EC}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (h_reg << 23) 'SET     5,H             CBEC            5b <- 1
{ED}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (l_reg << 23) 'SET     5,L             CBED            5b <- 1
{EE}                    long    vector_lmm     + (set_mem_lmm_no      << 9) + (bit_5_mask      << 15)                 'SET     5,(HL)          CBEE            5b <- 1
{EF}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_5_mask      << 15) + (a_reg << 23) 'SET     5,A             CBEF            5b <- 1

{F0}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (b_reg << 23) 'SET     6,B             CBF0            6b <- 1
{F1}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (c_reg << 23) 'SET     6,C             CBF1            6b <- 1
{F2}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (d_reg << 23) 'SET     6,D             CBF2            6b <- 1
{F3}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (e_reg << 23) 'SET     6,E             CBF3            6b <- 1
{F4}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (h_reg << 23) 'SET     6,H             CBF4            6b <- 1
{F5}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (l_reg << 23) 'SET     6,L             CBF5            6b <- 1
{F6}                    long    vector_lmm     + (set_mem_lmm_no      << 9) + (bit_6_mask      << 15)                 'SET     6,(HL)          CBF6            6b <- 1
{F7}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_6_mask      << 15) + (a_reg << 23) 'SET     6,A             CBF7            6b <- 1
{F8}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (b_reg << 23) 'SET     7,B             CBF8            7b <- 1
{F9}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (c_reg << 23) 'SET     7,C             CBF9            7b <- 1
{FA}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (d_reg << 23) 'SET     7,D             CBFA            7b <- 1
{FB}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (e_reg << 23) 'SET     7,E             CBFB            7b <- 1
{FC}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (h_reg << 23) 'SET     7,H             CBFC            7b <- 1
{FD}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (l_reg << 23) 'SET     7,L             CBFD            7b <- 1
{FE}                    long    vector_lmm     + (set_mem_lmm_no      << 9) + (bit_7_mask      << 15)                 'SET     7,(HL)          CBFE            7b <- 1
{FF}                    long    vector_lmm     + (set_reg_lmm_no      << 9) + (bit_7_mask      << 15) + (a_reg << 23) 'SET     7,A             CBFF            7b <- 1
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
dispatch_table_dd
{00}                    long    undocumented
{01}                    long    undocumented
{02}                    long    undocumented
{03}                    long    undocumented
{04}                    long    undocumented
{05}                    long    undocumented
{06}                    long    undocumented
{07}                    long    undocumented
{08}                    long    undocumented                                
{09}                    long    get_bc             + (double_add_ixy      << 9) + (put_ixy        << 18)              '* ADD     IX,BC           DD09            IX <- IX + BC
{0A}                    long    undocumented
{0B}                    long    undocumented
{0C}                    long    undocumented
{0D}                    long    undocumented
{0E}                    long    undocumented
{0F}                    long    undocumented

{10}                    long    undocumented
{11}                    long    undocumented
{12}                    long    undocumented
{13}                    long    undocumented
{14}                    long    undocumented
{15}                    long    undocumented
{16}                    long    undocumented
{17}                    long    undocumented
{18}                    long    undocumented
{19}                    long    get_de             + (double_add_ixy      << 9) + (put_ixy        << 18)              '* ADD     IX,DE           DD19            IX <- IX + DE
{1A}                    long    undocumented
{1B}                    long    undocumented
{1C}                    long    undocumented
{1D}                    long    undocumented
{1E}                    long    undocumented
{1F}                    long    undocumented

{20}                    long    undocumented
{21}                    long    get_immediate_word + (put_ixy             << 9)                                       '* LD      IX,word         DD21word        IX <- word
{22}                    long    get_ixy            + (store_word_direct   << 9)                                       '* LD      (word),IX       DD22word        (word) <- IX
{23}                    long    get_ixy            + (increment_word      << 9) + (put_ixy        << 18)              '* INC     IX              DD23            IX <- IX + 1
{24}                    long    undocumented
{25}                    long    undocumented
{26}                    long    undocumented
{27}                    long    undocumented
{28}                    long    undocumented
{29}                    long    get_ixy            + (double_add_ixy      << 9) + (put_ixy        << 18)              '* ADD     IX,IX           DD29            IX <- IX + IX
{2A}                    long    load_word_direct   + (put_ixy             << 9)                                       '* LD      IX,(word)       DD2Aword        IX <- (word)
{2B}                    long    get_ixy            + (decrement_word      << 9) + (put_ixy        << 18)              '* DEC     IX              DD2B            IX <- IX - 1
{2C}                    long    undocumented
{2D}                    long    undocumented
{2E}                    long    undocumented
{2F}                    long    undocumented

{30}                    long    undocumented
{31}                    long    undocumented
{32}                    long    undocumented
{33}                    long    undocumented
{34}                    long    get_mem_xy_disp    + (increment_byte      << 9) + (put_memory_addr<< 18)              '** INC     (IX+disp)      DD34disp       (IX+disp) <- (IX+disp) + 1
{35}                    long    get_mem_xy_disp    + (decrement_byte      << 9) + (put_memory_addr<< 18)              '* DEC     (IX+disp)      DD35disp       (IX+disp) <- (IX+disp) - 1
{36}                    long    ld_ix_disp_byte                                                                       'LD      (IX+disp),byte DD76dispbyte   (IX+disp) <- byte
{37}                    long    undocumented
{38}                    long    undocumented
{39}                    long    get_sp             + (double_add_ixy      << 9) + (put_ixy        << 18)              '** ADD     IX,SP           DD39            IX <- IX + SP
{3A}                    long    undocumented
{3B}                    long    undocumented
{3C}                    long    undocumented
{3D}                    long    undocumented
{3E}                    long    undocumented
{3F}                    long    undocumented

{40}                    long    undocumented
{41}                    long    undocumented
{42}                    long    undocumented
{43}                    long    undocumented
{44}                    long    undocumented
{45}                    long    undocumented
{46}                    long    get_mem_xy_disp    + (put_b               << 9)                                       '** LD      B,(IX+disp)    DD46disp       B <- (IX+disp)
{47}                    long    undocumented
{48}                    long    undocumented
{49}                    long    undocumented
{4A}                    long    undocumented
{4B}                    long    undocumented
{4C}                    long    undocumented
{4D}                    long    undocumented
{4E}                    long    get_mem_xy_disp    + (put_c               << 9)                                       '* LD      C,(IX+disp)    DD4Edisp       C <- (IX+disp)
{4F}                    long    undocumented

{50}                    long    undocumented
{51}                    long    undocumented
{52}                    long    undocumented
{53}                    long    undocumented
{54}                    long    undocumented
{55}                    long    undocumented
{56}                    long    get_mem_xy_disp    + (put_d               << 9)                                       '* LD      D,(IX+disp)    DD56disp       D <- (IX+disp)
{57}                    long    undocumented
{58}                    long    undocumented
{59}                    long    undocumented
{5A}                    long    undocumented
{5B}                    long    undocumented
{5C}                    long    undocumented
{5D}                    long    undocumented
{5E}                    long    get_mem_xy_disp    + (put_e               << 9)                                       '* LD      E,(IX+disp)    DD5Edisp       E <- (IX+disp)
{5F}                    long    undocumented

{60}                    long    undocumented
{61}                    long    undocumented
{62}                    long    undocumented
{63}                    long    undocumented
{64}                    long    undocumented
{65}                    long    undocumented
{66}                    long    get_mem_xy_disp    + (put_h               << 9)                                       '* LD      H,(IX+disp)    DD66disp       H <- (IX+disp)
{67}                    long    undocumented
{68}                    long    undocumented
{69}                    long    undocumented
{6A}                    long    undocumented
{6B}                    long    undocumented
{6C}                    long    undocumented
{6D}                    long    undocumented
{6E}                    long    get_mem_xy_disp    + (put_l               << 9)                                       '* LD      L,(IX+disp)    DD6Edisp       L <- (IX+disp)
{6F}                    long    undocumented

{70}                    long    get_b              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),B    DD70disp       (IX+disp) <- B
{71}                    long    get_c              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),C    DD71disp       (IX+disp) <- C
{72}                    long    get_d              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),D    DD72disp       (IX+disp) <- D
{73}                    long    get_e              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),E    DD73disp       (IX+disp) <- E
{74}                    long    get_h              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),H    DD74disp       (IX+disp) <- H
{75}                    long    get_l              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),L    DD75disp       (IX+disp) <- L
{76}                    long    ld_ix_disp_byte                                                                       'LD      (IX+disp),byte DD76dispbyte   (IX+disp) <- byte
{77}                    long    get_a              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),A    DD77disp       (IX+disp) <- A
{78}                    long    undocumented
{79}                    long    undocumented
{7A}                    long    undocumented
{7B}                    long    undocumented
{7C}                    long    undocumented
{7D}                    long    undocumented
{7E}                    long    get_mem_xy_disp    + (put_a               << 9)                                       '* LD      A,(IX+disp)    DD7Edisp       A <- (IX+disp)
{7F}                    long    undocumented

{80}                    long    undocumented
{81}                    long    undocumented
{82}                    long    undocumented
{83}                    long    undocumented
{84}                    long    undocumented
{85}                    long    undocumented
{86}                    long    get_mem_xy_disp    + (alu_add             << 9) + (put_a          << 18)              '* ADD     A,(IX+disp)    DD86disp       A <- A + (IX+disp)
{87}                    long    undocumented
{88}                    long    undocumented
{89}                    long    undocumented
{8A}                    long    undocumented
{8B}                    long    undocumented
{8C}                    long    undocumented
{8D}                    long    undocumented
{8E}                    long    get_mem_xy_disp    + (alu_add_with_carry  << 9) + (put_a          << 18)              '* ADC     A,(IX+disp)    DD8Edisp       A <- A + (IX+disp) + Carry
{8F}                    long    undocumented

{90}                    long    undocumented
{91}                    long    undocumented
{92}                    long    undocumented
{93}                    long    undocumented
{94}                    long    undocumented
{95}                    long    undocumented
{96}                    long    get_mem_xy_disp    + (alu_sub             << 9) + (put_a          << 18)              '* SUB     (IX+disp)      DD96disp       A <- A - (IX+disp)
{97}                    long    undocumented
{98}                    long    undocumented
{99}                    long    undocumented
{9A}                    long    undocumented
{9B}                    long    undocumented
{9C}                    long    undocumented
{9D}                    long    undocumented
{9E}                    long    get_mem_xy_disp    + (alu_sub_with_borrow << 9) + (put_a          << 18)              '* SBC     (IX+disp)      DD9Edisp       A <- A - (IX+disp) - Carry
{9F}                    long    undocumented

{A0}                    long    undocumented
{A1}                    long    undocumented
{A2}                    long    undocumented
{A3}                    long    undocumented
{A4}                    long    undocumented
{A5}                    long    undocumented
{A6}                    long    get_mem_xy_disp    + (alu_and             << 9) + (put_a          << 18)              '* AND     (IX+disp)      DDA6disp       A <- A AND (IX+disp)
{A7}                    long    undocumented
{A8}                    long    undocumented
{A9}                    long    undocumented
{AA}                    long    undocumented
{AB}                    long    undocumented
{AC}                    long    undocumented
{AD}                    long    undocumented
{AE}                    long    get_mem_xy_disp    + (alu_xor             << 9) + (put_a          << 18)              '* XOR     (IX+disp)      DDAEdisp       A <- A XOR (IX+disp)
{AF}                    long    undocumented

{B0}                    long    undocumented
{B1}                    long    undocumented
{B2}                    long    undocumented
{B3}                    long    undocumented
{B4}                    long    undocumented
{B5}                    long    undocumented
{B6}                    long    get_mem_xy_disp    + (alu_or              << 9) + (put_a          << 18)              '* OR      (IX+disp)      DDB6disp       A <- A OR (IX+disp)
{B7}                    long    undocumented
{B8}                    long    undocumented
{B9}                    long    undocumented
{BA}                    long    undocumented
{BB}                    long    undocumented
{BC}                    long    undocumented
{BD}                    long    undocumented
{BE}                    long    get_mem_xy_disp    + (alu_sub             << 9) + (fetch          << 18)              '* CP      (IX+disp)      DDBEdisp       A - (IX+disp)
{BF}                    long    undocumented

{C0}                    long    undocumented
{C1}                    long    undocumented
{C2}                    long    undocumented
{C3}                    long    undocumented
{C4}                    long    undocumented
{C5}                    long    undocumented
{C6}                    long    undocumented
{C7}                    long    undocumented
{C8}                    long    undocumented
{C9}                    long    undocumented
{CA}                    long    undocumented
{CB}                    long    get_mem_xy_disp    + (z80_ddcb            << 9)                                       'Instruction ddcb prefix
{CC}                    long    undocumented
{CD}                    long    undocumented
{CE}                    long    undocumented
{CF}                    long    undocumented

{D0}                    long    undocumented
{D1}                    long    undocumented
{D2}                    long    undocumented
{D3}                    long    undocumented
{D4}                    long    undocumented
{D5}                    long    undocumented
{D6}                    long    undocumented
{D7}                    long    undocumented
{D8}                    long    undocumented
{D9}                    long    undocumented
{DA}                    long    undocumented
{DB}                    long    undocumented
{DC}                    long    undocumented
{DD}                    long    undocumented
{DE}                    long    undocumented
{DF}                    long    undocumented

{E0}                    long    undocumented
{E1}                    long    pop_               + (put_ixy             << 9)                                       'POP IX     DDE1
{E2}                    long    undocumented
{E3}                    long    not_implemented                                                                       'EX      (SP),IX         DDE3            IXh <-> (SP+1); IXl <-> (SP)
{E4}                    long    undocumented
{E5}                    long    get_ixy            + (push_               << 9)                                       'PUSH    IX              DDE5            (SP-2) <- IXl; (SP-1) <- IXh 
{E6}                    long    undocumented
{E7}                    long    undocumented
{E8}                    long    undocumented
{E9}                    long    get_ixy            + (put_pc              << 9)                                       '* JP      (IX)            DDE9            PC <- IX
{EA}                    long    undocumented
{EB}                    long    undocumented
{EC}                    long    undocumented
{ED}                    long    undocumented
{EE}                    long    undocumented
{EF}                    long    undocumented

{F0}                    long    undocumented
{F1}                    long    undocumented
{F2}                    long    undocumented
{F3}                    long    undocumented
{F4}                    long    undocumented
{F5}                    long    undocumented
{F6}                    long    undocumented
{F7}                    long    undocumented
{F8}                    long    undocumented
{F9}                    long    get_ixy            + (put_sp              << 9)                                       '* LD      SP,IX           DDF9            SP <- IX
{FA}                    long    undocumented
{FB}                    long    undocumented
{FC}                    long    undocumented
{FD}                    long    undocumented
{FE}                    long    undocumented
{FF}                    long    undocumented
'---------------------------------------------------------------------------------------------------------


'---------------------------------------------------------------------------------------------------------
dispatch_table_ddcb
{06}                    long    vector_lmm          + (0                   << 9) + (put_memory_sa  << 18)              'RLC     (IX+disp)      DDCBdisp06     ---
{0E}                    long    vector_lmm          + (0                   << 9) + (put_memory_sa  << 18)              'RRC     (IX+disp)      DDCBdisp0E     ---
{16}                    long    vector_lmm          + (0                   << 9) + (put_memory_sa  << 18)              'RL      (IX+disp)      DDCBdisp16     ---
{1E}                    long    vector_lmm          + (0                   << 9) + (put_memory_sa  << 18)              'RL      (IX+disp)      DDCBdisp1E     ---
{26}                    long    vector_lmm          + (0                   << 9) + (put_memory_sa  << 18)              'SLA     (IX+disp)      DDCBdisp26     ---
{2E}                    long    vector_lmm          + (0                   << 9) + (put_memory_sa  << 18)              'SRA     (IX+disp)      DDCBdisp2E     ---
{36}                    long    undocumented
{3E}                    long    vector_lmm         + (not_implemented         << 9)                                    'SRL     (IX+disp)      DDCBdisp3E     ---
{46}                    long    vector_lmm         + (bit_idx_lmm            << 9)                                    'BIT     0,(IX+disp)    DDCBdisp46     Z flag <- NOT 0b
{4E}                    long    vector_lmm         + (bit_idx_lmm            << 9)                                    'BIT     1,(IX+disp)    DDCBdisp4E     Z flag <- NOT 1b
{56}                    long    vector_lmm         + (bit_idx_lmm            << 9)                                    'BIT     2,(IX+disp)    DDCBdisp56     Z flag <- NOT 2b
{5E}                    long    vector_lmm         + (bit_idx_lmm            << 9)                                    'BIT     3,(IX+disp)    DDCBdisp5E     Z flag <- NOT 3b
{66}                    long    vector_lmm         + (bit_idx_lmm            << 9)                                    'BIT     4,(IX+disp)    DDCBdisp66     Z flag <- NOT 4b
{6E}                    long    vector_lmm         + (bit_idx_lmm            << 9)                                    'BIT     5,(IX+disp)    DDCBdisp6E     Z flag <- NOT 5b
{76}                    long    vector_lmm         + (bit_idx_lmm            << 9)                                    'BIT     6,(IX+disp)    DDCBdisp76     Z flag <- NOT 6b
{7E}                    long    vector_lmm         + (bit_idx_lmm            << 9)                                    'BIT     7,(IX+disp)    DDCBdisp7E     Z flag <- NOT 7b
{86}                    long    vector_lmm         + (res_idx_lmm            << 9)                                    'RES     0,(IX+disp)    DDCBdisp86     0b <- 0
{8E}                    long    vector_lmm         + (res_idx_lmm            << 9)                                    'RES     1,(IX+disp)    DDCBdisp8E     1b <- 0
{96}                    long    vector_lmm         + (res_idx_lmm            << 9)                                    'RES     2,(IX+disp)    DDCBdisp96     2b <- 0
{9E}                    long    vector_lmm         + (res_idx_lmm            << 9)                                    'RES     3,(IX+disp)    DDCBdisp9E     3b <- 0
{A6}                    long    vector_lmm         + (res_idx_lmm            << 9)                                    'RES     4,(IX+disp)    DDCBdispA6     4b <- 0
{AE}                    long    vector_lmm         + (res_idx_lmm            << 9)                                    'RES     5,(IX+disp)    DDCBdispAE     5b <- 0
{B6}                    long    vector_lmm         + (res_idx_lmm            << 9)                                    'RES     6,(IX+disp)    DDCBdispB6     6b <- 0
{BE}                    long    vector_lmm         + (res_idx_lmm            << 9)                                    'RES     7,(IX+disp)    DDCBdispBE     7b <- 0
{C6}                    long    vector_lmm         + (set_idx_lmm            << 9)                                    'SET     0,(IX+disp)    DDCBdispC6     0b <- 1
{CE}                    long    vector_lmm         + (set_idx_lmm            << 9)                                    'SET     1,(IX+disp)    DDCBdispCE     1b <- 1
{D6}                    long    vector_lmm         + (set_idx_lmm            << 9)                                    'SET     2,(IX+disp)    DDCBdispD6     2b <- 1
{DE}                    long    vector_lmm         + (set_idx_lmm            << 9)                                    'SET     3,(IX+disp)    DDCBdispDE     3b <- 1
{E6}                    long    vector_lmm         + (set_idx_lmm            << 9)                                    'SET     4,(IX+disp)    DDCBdispE6     4b <- 1
{EE}                    long    vector_lmm         + (set_idx_lmm            << 9)                                    'SET     5,(IX+disp)    DDCBdispEE     5b <- 1
{F6}                    long    vector_lmm         + (set_idx_lmm            << 9)                                    'SET     6,(IX+disp)    DDCBdispF6     6b <- 1
{FE}                    long    vector_lmm         + (set_idx_lmm            << 9)                                    'SET     7,(IX+disp)    DDCBdispFE     7b <- 1
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
dispatch_table_ed
{40}                    long    in_c               + (put_b               << 9)                           '*IN      B,(C)           ED40            B <- [C]
{41}                    long    get_b              + (out_c               << 9)                           '*OUT     (C),B           ED41            [C] <- B
{42}                    long    get_bc             + (double_sub_hl_cy    << 9)                           '*SBC     HL,BC           ED42            HL <- HL - BC - Carry
{43}                    long    get_bc             + (store_word_direct   << 9)                           '*LD      (word),BC       ED43word        (word) <- BC
{44}                    long    vector_lmm     + (alu_neg_lmm_no      << 9)                           '*NEG                     ED44            A <- 0-A
{45}                    long    not_implemented                                                           'RETN                    ED45            IFF1 <- IFF2;RETI
{46}                    long    not_implemented                                                           'IM      0               ED46            Set interrupt mode 0
{47}                    long    not_implemented                                                           'LD      I,A             ED47            Interrupt Page <- A
{48}                    long    in_c               + (put_c               << 9)                           '*IN      C,(C)           ED48            C <- [C]
{49}                    long    get_c              + (out_c               << 9)                           '*OUT     (C),C           ED49            [C] <- C
{4A}                    long    get_bc             + (double_add_hl_cy    << 9)                           '*ADC     HL,BC           ED4A            HL <- HL + BC + Carry
{4B}                    long    load_word_direct   + (put_bc              << 9)                           '*LD      BC,(word)       ED4Bword        BC <- (word)
{4C}                    long    undocumented
{4D}                    long    not_implemented                                                           'RETI                    ED4D            Return from Interrupt
{4E}                    long    undocumented
{4F}                    long    not_implemented                                                          'LD       R,A             ED4F           Refresh counter <- A

{50}                    long    in_c               + (put_d               << 9)                           '*IN      D,(C)           ED50            D <- [C]
{51}                    long    get_d              + (out_c               << 9)                           '*OUT     (C),D           ED51            [C] <- D
{52}                    long    get_de             + (double_sub_hl_cy    << 9)                           '*SBC     HL,DE           ED52            HL <- HL - DE - Carry
{53}                    long    get_de             + (store_word_direct   << 9)                           'LD      (word),DE       ED53word        (word) <- DE
{54}                    long    undocumented
{55}                    long    undocumented
{56}                    long    get_mem_xy_disp    + (put_d               << 9)                           'IM      1               ED56            Set interrupt mode 0
{57}                    long    undocumented                                                              'LD       A,I             ED57            A <- Interrupt Page ?? 
{58}                    long    in_c               + (put_e               << 9)                           '*IN      E,(C)           ED58            E <- [C]
{59}                    long    get_e              + (out_c               << 9)                           '*OUT     (C),E           ED59            [C] <- E
{5A}                    long    get_de             + (double_add_hl_cy    << 9)                           '*ADC     HL,DE           ED5A            HL <- HL + DE + Carry
{5B}                    long    load_word_direct   + (put_de              << 9)                           'LD      DE,(word)       ED5Bword        DE <- (word)
{5C}                    long    undocumented
{5D}                    long    undocumented
{5E}                    long    not_implemented                                                           'IM       2               ED5E           Set interrupt mode 0
{5F}                    long    not_implemented                                                           'LD       A,R             ED5F           A <- Refresh counter

{60}                    long    in_c               + (put_h               << 9)                           '*IN      H,(C)           ED60            H <- [C]
{61}                    long    get_h              + (out_c               << 9)                           '*OUT     (C),H           ED61            [C] <- H
{62}                    long    get_hl             + (double_sub_hl_cy    << 9)                           '*SBC     HL,HL           ED62            HL <- HL - HL - Carry
{63}                    long    get_hl             + (store_word_direct   << 9)                           'LD      (word),HL       ED6Bword        (word) <- HL
{64}                    long    undocumented
{65}                    long    undocumented
{66}                    long    undocumented
{67}                    long    not_implemented                                                           'RRD                     ED67            Rotate Right Digit
{68}                    long    in_c               + (put_l               << 9)                           '*IN      L,(C)           ED68            L <- [C]
{69}                    long    get_l              + (out_c               << 9)                           '*OUT     (C),L           ED69            [C] <- L
{6A}                    long    get_hl             + (double_add_hl_cy    << 9)                           '*ADC     HL,HL           ED6A            HL <- HL + HL + Carry
{6B}                    long    load_word_direct   + (put_hl              << 9)                           '*LD      HL,(word)       ED6Bword        HL <- (word)
{6C}                    long    undocumented
{6D}                    long    undocumented
{6E}                    long    undocumented
{6F}                    long    not_implemented                                                           'RLD                     ED6F            Rotate Left Digit

{70}                    long    undocumented
{71}                    long    undocumented
{72}                    long    get_sp             + (double_sub_hl_cy    << 9)                           '*SBC     HL,SP           ED72            HL <- HL - SP - Carry
{73}                    long    get_sp             + (store_word_direct   << 9)                           'LD      (word),SP       ED73word        (word) <- SP
{74}                    long    undocumented
{75}                    long    undocumented
{76}                    long    undocumented
{77}                    long    undocumented
{78}                    long    in_c               + (put_a               << 9)                           '*IN      A,(C)           ED78            A <- [C]
{79}                    long    get_a              + (out_c               << 9)                           '*OUT     (C),A           ED79            [C] <- A
{7A}                    long    get_sp             + (double_add_hl_cy    << 9)                           '*ADC     HL,SP           ED7A            HL <- HL + SP + Carry
{7B}                    long    load_word_direct   + (put_sp              << 9)                           'LD      SP,(word)        ED7Bword        SP <- (word)
{7C}                    long    undocumented
{7D}                    long    undocumented
{7E}                    long    undocumented
{7F}                    long    undocumented

{80}                    long    undocumented
{81}                    long    undocumented
{82}                    long    undocumented
{83}                    long    undocumented
{84}                    long    undocumented
{85}                    long    undocumented
{86}                    long    undocumented
{87}                    long    undocumented
{88}                    long    undocumented
{89}                    long    undocumented
{8A}                    long    undocumented
{8B}                    long    undocumented
{8C}                    long    undocumented
{8D}                    long    undocumented
{8E}                    long    undocumented
{8F}                    long    undocumented

{90}                    long    undocumented
{91}                    long    undocumented
{92}                    long    undocumented
{93}                    long    undocumented
{94}                    long    undocumented
{95}                    long    undocumented
{96}                    long    undocumented
{97}                    long    undocumented
{98}                    long    undocumented
{99}                    long    undocumented
{9A}                    long    undocumented
{9B}                    long    undocumented
{9C}                    long    undocumented
{9D}                    long    undocumented
{9E}                    long    undocumented
{9F}                    long    undocumented

{A0}                    long    vector_lmm     + (ldi_lmm_no          << 9)                           'LDI                     EDA0            (DE) <- (HL); HL <- HL+1; DE <- DE+1; BC <- BC-1
{A1}                    long    vector_lmm     + (cpi_lmm_no          << 9)                           'CPI                     EDA1            A - (HL); HL <- HL+1; BC <- BC-1
{A2}                    long    vector_lmm     + (ini_lmm_no          << 9)                           'INI                     EDA2            (HL) <- [C]; B <- B-1; HL <- HL+1
{A3}                    long    vector_lmm     + (outi_lmm_no         << 9)                           'OUTI                    EDA3            [C] <- (HL); B <- B-1; HL <- HL+1
{A4}                    long    undocumented
{A5}                    long    undocumented
{A6}                    long    undocumented
{A7}                    long    undocumented
{A8}                    long    vector_lmm     + (ldd_lmm_no          << 9)                           'LDD                     EDA8            (DE) <- (HL); HL <- HL-1; DE <- DE-1; BC <- BC-1
{A9}                    long    vector_lmm     + (cpd_lmm_no          << 9)                           'CPD                     EDA9            A - (HL); HL <- HL-1; BC <- BC-1
{AA}                    long    vector_lmm     + (ind_lmm_no          << 9)                           'IND                     EDAA            (HL) <- [C]; B <- B-1; HL <- HL-1
{AB}                    long    vector_lmm     + (outd_lmm_no         << 9)                           'OUTD                    EDAB            [C] <- (HL); B <- B-1; HL <- HL-1
{AC}                    long    undocumented
{AD}                    long    undocumented
{AE}                    long    undocumented
{AF}                    long    undocumented

{B0}                    long    vector_lmm     + (ldir_lmm_no         << 9)                           'LDIR                    EDB0            (DE) <- (HL); HL <- HL+1; DE <- DE+1; BC <- BC-1
{B1}                    long    vector_lmm     + (cpir_lmm_no         << 9)                           'CPIR                    EDB1            A - (HL); HL <- HL+1; BC <- BC-1
{B2}                    long    vector_lmm     + (inir_lmm_no         << 9)                           'INIR                    EDB2            (HL) <- [C]; B <- B-1; HL <- HL+1
{B3}                    long    vector_lmm     + (otir_lmm_no         << 9)                           'OTIR                    EDB3            [C] <- (HL); B <- B-1; HL <- HL+1
{B4}                    long    undocumented
{B5}                    long    undocumented
{B6}                    long    undocumented
{B7}                    long    undocumented
{B8}                    long    vector_lmm     + (lddr_lmm_no         << 9)                           'LDDR                    EDB8            (DE) <- (HL); HL <- HL-1; DE <- DE-1; BC <- BC-1
{B9}                    long    vector_lmm     + (cpdr_lmm_no         << 9)                           'CPDR                    EDB9            A - (HL); HL <- HL-1; BC <- BC-1
{BA}                    long    vector_lmm     + (indr_lmm_no         << 9)                           'INDR                    EDBA            (HL) <- [C]; B <- B-1; HL <- HL-1
{BB}                    long    vector_lmm     + (otdr_lmm_no         << 9)                           'OTDR                    EDBB            [C] <- (HL); B <- B-1; HL <- HL-1
'---------------------------------------------------------------------------------------------------------
#endif
'The end.
